;*******************************************************************************************
;   Beschreibung: DCF77 Uhr mit 8 Mhz Quarz, Berechnung der Sternzeit aus DCF Daten und 
;   				der Position, Eingabe der Wert ber Mens
;   Takt : 8 MHz
;
;								 *							 *
;*******************************************************************************************
;                                                                                          *
;    Dateiname:	 	DCF77_Uhr_P16F876A.asm						                               		 *
;    Datum:        	16.02.2015                                                            *
;    Version: 			V1.1                                                                  *
;                                                                                          *
;    Autor:       	Werner Hirsch                                                       *
;    Internet:      www.wernerhirsch.de                                                           *
;    e-mail:		wernerhirschhh@aol.com                                                          *
;                                                                                          *
;*******************************************************************************************
;    Bemerkungen:                                                                          *
;																														 *
;	Bankumschaltungen:
;	BANK_0:	BCF STATUS,RP0
;			BCF STATUS,RP1
;	BANK_1:	BSF STATUS,RP0
;			BCF STATUS,RP1
;	BANK_2:	BCF STATUS,RP0
;			BSF STATUS,RP1
;	BANK_3:	BSF STATUS,RP0
;			BSF STATUS,RP1														 *
;                                                                                          *
;*******************************************************************************************


	list      p=16f876a			;definiert den verwendeten Prozessor
	#include <p16f876a.inc>		;in diesem File stehen die Adressen und Bitnamen

	ERRORLEVEL      -302		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -305		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -306		;SUPPRESS BANK SELECTION MESSAGES
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_OFF & _HS_OSC & _WRT_OFF & _LVP_OFF & _DEBUG_OFF & _CPD_OFF 
	; Mit '__CONFIG' werden die Configurationsbits gesetzt, z.B. Watchdog an/aus, Debugmodus

;===========================
;  DEFINITIONEN
;===========================
#define IN_NC_A0		PORTA, 0		;PIN RA0 (pin 2)
#define IN_NC_A1		PORTA, 1		;PIN RA1 (pin 3)
#define IN_NC_A2		PORTA, 2		;PIN RA2 (pin 4)
#define DISP_RS			PORTA, 3		;Unterscheidung zwischen Befehl und Daten PIN RA3 (pin 5)
#define IN_DCF			PORTA, 4		;Eingang DCF77 Signal PIN RA4 (pin 6)
#define DISP_CLK		PORTA, 5		;Display Enable PIN RA5 (pin 7)
#define IN_TASTER		PORTB, 0		;Men PIN RB0 (pin 21)
#define IN_NC_B1		PORTB, 1		;OK PIN RB1 (pin 22)
#define IN_NC_B2		PORTB, 2		;Left PIN RB2 (pin 23)
#define IN_NC_B3		PORTB, 3		;Right RB3 (pin 24)
#define IN_NC_B4		PORTB, 4		;Up PIN RB4 (pin 25)
#define IN_NC_B5		PORTB, 5		;Down PIN RB5 (pin 26)
#define IN_NC_B6		PORTB, 6		;PIN RB6 (pin 27), PGC
#define IN_NC_B7		PORTB, 7		;PIN RB7 (pin 28), PGD
#define OUT_DB0			PORTC, 0		;Bit0 fr LCD Anzeige, PIN RC0 (pin 11)
#define OUT_DB1			PORTC, 1		;Bit1 fr LCD Anzeige, PIN RC1 (pin 12)
#define OUT_DB2			PORTC, 2		;Bit2 fr LCD Anzeige, PIN RC2 (pin 13)
#define OUT_DB3			PORTC, 3		;Bit3 fr LCD Anzeige, PIN RC3 (pin 14)
#define OUT_DB4			PORTC, 4		;Bit4 fr LCD Anzeige, PIN RC4 (pin 15)
#define OUT_DB5			PORTC, 5		;Bit5 fr LCD Anzeige, PIN RC5 (pin 16)
#define OUT_DB6			PORTC, 6		;Bit6 fr LCD Anzeige, PIN RC6 (pin 17)
#define OUT_DB7			PORTC, 7		;Bit7 fr LCD Anzeige, PIN RC7 (pin 18)



;===========================
;  KONSTANTEN
;===========================
HIGH_BYTE_S 	EQU 0x10		;Highbyte fr Sekundenzhler Uhrzeit
LOW_BYTE_S		EQU 0x42		;Lowbyte fr Sekundenzhler Uhrzeit
HIGH_BYTE_SZ 	EQU 0x10		;Highbyte fr Sekundenzhler Sternzeit
LOW_BYTE_SZ		EQU 0x38		;Lowbyte fr Sekundenzhler Sternzeit

;===========================
;  VARIABLEN
;===========================
w_temp        	EQU	0x7D        ; werden fr Interrupts verwendet, um den Status vor 
status_temp   	EQU	0x7E        ; der Interruptroutine zu sichern
pclath_temp		EQU	0x7F		;

COUNT_L_S		EQU	0x20		;Zhlregister fr Lowbyte Sekundenzhler Uhrzeit
COUNT_H_S		EQU	0x21		;Zhlregister fr Highbyte Sekundenzhler Uhrzeit
COUNT_N			EQU 0x22		;Zhlregister fr Anpassung Lowbyte Sekundenzhler Uhrzeit (nach n-1)
COUNT_L_SZ		EQU	0x23		;Zhlregister fr Lowbyte Sekundenzhler Sternzeit
COUNT_H_SZ		EQU	0x24		;Zhlregister fr Highbyte Sekundenzhler Sternzeit
COUNT_NSZ		EQU 0x25		;Zhlregister fr Anpassung Lowbyte Sekundenzhler Sternzeit (nach n-1)
SEK_UZ			EQU	0x26		;
MIN_UZ			EQU	0x27		;
STD_UZ			EQU	0x28		;
TAG_UZ			EQU 0x29		;
MONAT_UZ		EQU 0x2A		;
JAHR_UZ			EQU 0x2B		;
TEMP_11			EQU	0x2C		;Allgemeines Zwischenspeicherregister
TEMP_12			EQU	0x2D		;Allgemeines Zwischenspeicherregister
TEMP_13			EQU	0x2E		;Allgemeines Zwischenspeicherregister
TEMP_14			EQU	0x2F		;Allgemeines Zwischenspeicherregister
TEMP_15			EQU	0x30		;Allgemeines Zwischenspeicherregister
TEMP_16			EQU 0x31		;Allgemeines Zwischenspeicherregister
TEMP_17			EQU 0x32		;Allgemeines Zwischenspeicherregister
TEMP_18			EQU 0x33		;Allgemeines Zwischenspeicherregister
LOOP_1			EQU 0x34		;Zhler fr Wartezeit, allgemeiner temporrer Zhler, darf nicht in Interrupts verwendet werden
DISP_SHIFT_REG	EQU	0x35		;Datum, welches an das Display gesendet werden soll
DISP_SHIFT_CNT	EQU	0x36		;Zhler fr die Datenausgabe
DISP_NEU		EQU	0x37		;Statusregister Display:
								;Bit0: Normalzeituhr, Bit1: Sternzeituhr, Bit2: Datum, Bit3: -
								;Bit4: Menue 1.Stufe, Bit5: -, Bit6: -, Bit7: Anzeige Sternzeit oder Datum
DCF_L_MIN		EQU 0x38		;Min Lnge fr DCF Signalabsenkung "0" Bit
DCF_L_MAX		EQU 0x39		;Max Lnge fr DCF Signalabsenkung "0" Bit
DCF_H_MIN		EQU 0x3A		;Min Lnge fr DCF Signalabsenkung "1" Bit
DCF_H_MAX		EQU 0x3B		;Max Lnge fr DCF Signalabsenkung "1" Bit
DCF_JAHR		EQU	0x3C		;Register fr Daten vom DCF77 Empfnger
DCF_MONAT		EQU	0x3D		;Register fr Daten vom DCF77 Empfnger
DCF_TAG			EQU	0x3E		;Register fr Daten vom DCF77 Empfnger
DCF_WTAG		EQU	0x3F		;Register fr Daten vom DCF77 Empfnger, Wochentag,
DCF_STD			EQU	0x40		;Register fr Daten vom DCF77 Empfnger
DCF_MIN			EQU	0x41		;Register fr Daten vom DCF77 Empfnger
DCF_BIT			EQU	0x42		;Bit Zhler fr Daten vom DCF77 Empfnger
DCF_MESS_STAT	EQU	0x43		;Statusregister DCF77 Empfang: 
								;Bit0: Updateanforderung (Men oder Uhrzeit), Bit1: Fehlerbit, Bit2: Status DCF Eingang, 
								;Bit3: start des neuen DCF Telegramms, Bit4: neues Bit empfangen, 
								;Bit5: wird gesetzt,wenn Anzahl Low Bits =20, Bit6: wird gesetzt,wenn Anzahl High Bits =20,
TBIT_H			EQU	0x44		;High byte fr Zeit DCF77 Signalabsenkung
TBIT_L			EQU	0x45		;Low byte fr Zeit DCF77 Signalabsenkung
TBIT_C			EQU	0x46		;Vorteiler /4 fr Zeit DCF77 Signalabsenkung
DCF_BIT_TEMP	EQU	0x47		;Temp-Register fr Zwischenspeicherung des empfangenen DCF Bits 
DCF_MESZ		EQU 0x48		;Register fr bits MEZ/MESZ: 00000010=MESZ=2, 00000001=MEZ=1
DCF_PRUEFBIT	EQU 0x49		;Register fr Zwischenspeicherung Prfbit
DCF_COUNT		EQU 0x4A		;Zhler fr Anzahl fehlerhafter Versuche 
ZZ_UZ			EQU 0x4B		;Register fr Anzeige Zeitzone vs UTC, wenn ZZ negativ ist wird Bit 7 gesetzt
WTAG_UZ			EQU 0x4C		;Register fr Wochentagszahl, 1 = MO, 2 = DI.... 7 = SO
TBIT_H_TEMP		EQU	0x4D		;High byte fr Zeit DCF77 Signalabsenkung, Temp-Register
TBIT_L_TEMP		EQU	0x4E		;Low byte fr Zeit DCF77 Signalabsenkung, Temp-Register
BEFEHL			EQU 0x4F		;Register fr Tastenbefehl
								;bit 0 = Menue		bit 1 = OK		bit 2 = Left		bit 3 = Right
								;bit 4 = Up			bit 5 = Down	bit 6 = 
MENUE_FF		EQU 0x50		;Statusregister: bit 0 = Menuemodus, 
MENUE_COUNT		EQU 0x51		;Zhler fr Menposition vertikal
MENUE_1_CNT		EQU 0x52		;Zhler
TEMP_DIG_CNT	EQU 0x53		;Zhler fr Menposition horizontal
TEMP_CNT_MAX	EQU 0x54		;Max Zhlerwert fr Menposition horizontal
TEMP_DIG_ADR	EQU 0x55		;Display Adressregister fr das zu ndernde Digit
TEMP_DIG_VAL	EQU 0x56		;Display Wert fr das zu ndernde Digit
TEMP_VAL_PREV	EQU 0x57		;Display Wert fr das vorhergehende zu ndernde Digit
TEMP_DIG_STAT	EQU 0x58		;Status Register	bit 0: wenn 1: Update der Anzeige des zu ndernden Wertes
								;					bit 1: wenn 1: Erhhung des zu ndernden Wertes
								;					bit 2: wenn 1: Reduzierung des zu ndernden Wertes
MESS_END		EQU 0x59		;Status Register	bit 0: wenn 1: 
								;					bit 1: wenn 1: 
								;					bit 2: wenn 1: 
TMR0_TEMP		EQU 0x5A		;Temp Register zur Zwischenspeicherung des Timer 0 Wertes In der Interrupt Routine
;Register fr Sternzeitberechnung
SEK_SZ			EQU	0x60		;
MIN_SZ			EQU	0x61		;
STD_SZ			EQU	0x62		;
JAHR_4			EQU	0x63		;Register fr Jahr: Schaltjahr =0, SJ+1 =1, SJ+2 =2, SJ+3 =3
TAGE_L			EQU	0x64		;Low Byte Register fr Anzahl abgelaufener Tage im aktuellen Jahr
TAGE_H			EQU	0x65		;High Byte Register fr Anzahl abgelaufener Tage im aktuellen Jahr
STD_TEMP		EQU	0x66		;
MIN_TEMP		EQU	0x67		;
SEK_TEMP		EQU	0x68		;
REST_SEK_TEMP	EQU	0x69		;2 Nachkommastellen fr Sekundenberechnung Sternzeit
MIN_TEM_01		EQU	0x6A		;Register fr Winkelsekunden
MIN_TEM			EQU	0x6B		;Register fr Winkelminuten
GRD_TEM			EQU 0x6C		;Register fr WinkelGrad
PM_TEM			EQU 0x6D		;Register fr Vorzeichen +/-

;__________________________________________________________________________________________
;
		ORG 	H'2110'			;Adresse des EEPROM fr den Brenner (2100 = erster Speicherplatz)
;Konstanten Position
		de		0x00			;Adresse 10,	Ost=0, West=1
		de		0x09			;Adresse 11,	Winkelgrad
		de		0x35			;Adresse 12,	Winkelminuten 
		de		0x24			;Adresse 13,	Winkelsekunden
;Konstanten Update DCF Zeit
		de		0x03			;Adresse 14,	Bit 0-6 = Zeit (00h...23h), Bit 7: 0 = auto, 1 = manuell
;Konstanten zulssige Min/Max Bitlngen des DCF Zeitsignals
		de		0x44			;Adresse 15,	Low  Bit min ("0" Bit):  70ms (Zhlerwert: 68)
		de		0x7E			;Adresse 16,	Low  Bit max ("0" Bit): 130ms (Zhlerwert: 126)
		de		0xA6			;Adresse 17,	High Bit min ("1" Bit): 170ms (Zhlerwert: 166)
		de		0xE0			;Adresse 18,	High Bit max ("1" Bit): 230ms (Zhlerwert: 224)



;__________________________________________________________________________________________

		ORG     0			; Adresse, an die nach einem Reset gesprungen wird
		clrf    PCLATH          ; Rcksetzen der Page-Bits
  		goto    Start           ; Springe zum Programmanfang

		ORG     4           ; Adresse, an die nach einem Interrupt gesprungen wird
		movwf	w_temp          ; Sichere den Inhalt des W-Registers
		movf    STATUS,w        ; Verschiebe STATUS-Register ins W-Register
		movwf	status_temp     ; Sichere den Inhalt des STATUS-Registers
		movf    PCLATH,w        ; Verschiebe PCLATH-Register ins W-Register
		movwf	pclath_temp     ; Sichere den Inhalt des PCLATH-Registers


Int_serv 						;Interrupt Routine
	bcf PCLATH,3				;Select Page 0
	bcf PCLATH,4				;
	bsf MESS_END,2				;setze Statusbit fr Timer 0 berlauf
	btfss DCF_MESS_STAT,0		;prfen ob bit fr DCF77 Empfang gesetzt ist
	goto Uhr_Intern				;wenn ja bspringe den nchsten Befehl, wenn nein weiter bei Uhr_Intern
	decfsz TBIT_C				;reduziere 0,256ms Vorteiler und prfe ob 0
	goto Uhr_Intern
	movlw 0x04					;wenn 0, Rcksetzen des Zhlers mit Wert 4
	movwf TBIT_C
	incf TBIT_L					;erhhe 1,024ms Zhler fr Low byte und prfe ob 0
	movfw TBIT_L				;
	btfsc STATUS,Z
	incf TBIT_H					;erhhe 262ms Zhler
	movlw d'5'					;prfen ob zhler 5 erreicht ist (entspricht 1,311sek)
	subwf TBIT_H,w
	btfsc STATUS,Z
	bsf DCF_MESS_STAT,3			;wenn ja setze statusbit fr neues Datentelegramm (Bit59)

	btfss IN_DCF				;wenn Eingang high, berspringe den nchsten Befehl	
	goto test_low				;
;wenn nach 5sek keine negative Flanke ermittelt wurde, wird das Fehlerbit gesetzt und
;das bit fr Ende Bit Auswertung gesetzt, um die Warteschleife abzubrechen
	movlw d'19'					;prfen ob zhler 19 erreicht ist (entspricht 5sek)
	subwf TBIT_H,w
	btfss STATUS,Z
	goto $+3
	bsf DCF_MESS_STAT,1			;wenn ja setze Fehlerbit und Bit fr Ende Bit Auswertung
	bsf DCF_MESS_STAT,4			;
	btfsc DCF_MESS_STAT,2		;prfen ob Bit fr high Position des DCF Eingangs nicht gesetzt ist
	goto Uhr_Intern				;wenn ja, berspringe diesen Befehl, (negative Flanke)
	bsf DCF_MESS_STAT,2			;setze bit fr high Position des DCF Eingangs
	;Auswertung der Zeit
	movfw TBIT_L
	movwf TBIT_L_TEMP
	movfw TBIT_H
	movwf TBIT_H_TEMP
	clrf DCF_BIT_TEMP
	bcf DCF_MESS_STAT,1			;lsche Fehlerbit
	movfw DCF_H_MAX				;prfen ob TBIT_L > DCF_H_MAX, wenn ja gehe zu fehlerbit
	subwf TBIT_L,w
	btfsc STATUS,C
	goto fehlerbit
	movfw DCF_H_MIN				;prfen ob TBIT_L < DCF_H_MIN, wenn nein setze bit fr "1", 
	subwf TBIT_L,w	
	btfss STATUS,C
	goto $+3
	bsf DCF_BIT_TEMP,0
	goto auswertbit_end			;Bit ist gltig und "1"
	movfw DCF_L_MAX				;prfen ob TBIT_L > DCF_L_MAX, wenn ja gehe zu fehlerbit
	subwf TBIT_L,w
	btfsc STATUS,C
	goto fehlerbit
	movfw DCF_L_MIN				;prfen ob TBIT_L < DCF_L_MIN, wenn nein lsche bit fr "0", 
	subwf TBIT_L,w
	btfss STATUS,C
	goto fehlerbit
	bcf DCF_BIT_TEMP,0
	goto auswertbit_end			;Bit ist gltig und "0"
auswertbit_end
	bsf DCF_MESS_STAT,4			;setze Bit fr Ende Bit Auswertung
	goto Uhr_Intern
fehlerbit
	bsf DCF_MESS_STAT,1			;setze Fehlerbit
	bsf DCF_MESS_STAT,4			;setze Bit fr Ende Bit Auswertung
	goto Uhr_Intern	
test_low
	btfss DCF_MESS_STAT,2		;prfen ob Bit fr high Position des DCF Eingangs gesetzt ist
	goto Uhr_Intern				;wenn ja, berspringe diesen Befehl, (negative Flanke)
	bcf DCF_MESS_STAT,2			;lsche Bit fr high Position des DCF Eingangs
	clrf TBIT_L					;rcksetzen der Zhler
	clrf TBIT_H					;
	movlw 0x04
	movwf TBIT_C
	bcf DCF_MESS_STAT,1
Uhr_Intern
	decfsz COUNT_L_S,f			;Reduziere Zhler fr Low Byte und prfe ob 0
	goto Uhr_Sternzeit			;wenn ja berspringe diesen Befehl, wenn nein Rcksprung aus dem Interupt
	decfsz COUNT_H_S,f			;Reduziere Zhler fr High Byte und prfe ob 0
	goto Uhr_Sternzeit			;Sekunde noch nicht vorbei
	; eine Sekunde ist vorbei Sekundentimer wieder einstellen
	bsf MESS_END,0				;Setze Statusbit fr neue Sekunde
	movlw HIGH_BYTE_S
	movwf COUNT_H_S				;
	movlw LOW_BYTE_S
	movwf COUNT_L_S
	decfsz COUNT_N,f			;Reduziere Zhler und prfe ob 0
	goto $+4					;Zhler >0<
	incf COUNT_L_S				;Zhler ist 0, Low byte einmalig um 1 vergrern und Zhler zurcksetzen mit Wert 4
	movlw 0x04
	movwf COUNT_N

	call inc_uz					;erhhe Normalzeit um 1 sek

Uhr_Sternzeit
	decfsz COUNT_L_SZ,f			;Reduziere Zhler fr Low byte und prfe ob 0
	goto Int_end				;wenn ja berspringe diesen Befehl, wenn nein Rcksprung aus dem Interupt
	decfsz COUNT_H_SZ,f			;Reduziere Zhler fr High byte und prfe ob 0
	goto Int_end				;Sekunde noch nicht vorbei
	; eine Sekunde ist vorbei Sekundentimer wieder einstellen
	movlw HIGH_BYTE_SZ
	movwf COUNT_H_SZ			;
	movlw LOW_BYTE_SZ
	movwf COUNT_L_SZ
	decfsz COUNT_NSZ,f			;Reduziere Zhler und prfe ob 0
	goto $+8					;Zhler >0<
	decf COUNT_L_SZ				;Zhler ist 0, Low byte einmalig um 5 verringern und Zhler zurcksetzen mit Wert 12
	decf COUNT_L_SZ
	decf COUNT_L_SZ
	decf COUNT_L_SZ
	decf COUNT_L_SZ
	movlw 0x0C
	movwf COUNT_NSZ

	call inc_sz					;erhhe Sternzeit um 1 sek

Int_end	
	bcf INTCON,T0IF				;TMR0 Interrupt flag lschen


		movf    pclath_temp,w     	; hole die Kopie des PCLATH-Registers
		movwf	PCLATH            	; Sichere die Daten wieder zurck in das PCLATH-Register
		movf    status_temp,w     	; hole die Kopie des STATUS-Registers
		movwf	STATUS            	; Sichere die Daten wieder zurck in das STATUS-Register
		swapf   w_temp,f
		swapf   w_temp,w          	; Sichere die Daten wieder zurck in das W-Register
		retfie                    	; Rcksprung vom Interrupt
;__________________________________________________________________________________________

Start						;Beginn des Programms
	bcf STATUS,RP0			;Select BANK_0
	bcf STATUS,RP1
	clrf PORTA				;Lsche alle Ausgangsports
	clrf PORTB
	clrf PORTC
	bsf STATUS,RP0			;Select BANK_1
	movlw b'00000111'		;alle Pins von Port A sind digitale Ein- oder Ausgnge
	movwf ADCON1
	movlw b'11010111'		;setze RA3 und RA5 als Ausgang, den Rest als Eingang
	movwf TRISA
	movlw b'11111111'		;setze RB0 bis RB7 als Eingang
	movwf TRISB
	movlw b'00000000'		;setze RC0 bis RC7 als Ausgang
	movwf TRISC
	movlw b'00000000'		;Pull Up Port B enable, TMR0 intern(Prescaler 1:2 (00000000)
	movwf OPTION_REG
	bcf STATUS,RP0			;Select BANK_0

	movlw HIGH_BYTE_S		;Rcksetzen der Sekundenzhler Uhrzeit+Sternzeit
	movwf COUNT_H_S
	movlw LOW_BYTE_S
	movwf COUNT_L_S
	movlw 0x04
	movwf COUNT_N
	movlw HIGH_BYTE_SZ
	movwf COUNT_H_SZ
	movlw LOW_BYTE_SZ
	movwf COUNT_L_SZ
	movlw 0x0C
	movwf COUNT_NSZ

	clrf DCF_MESS_STAT		;Lschen der Register
	clrf DCF_PRUEFBIT
	clrf DCF_COUNT
	clrf TBIT_H
	clrf TBIT_L
	clrf DCF_MIN
	clrf DCF_STD
	clrf DCF_JAHR
	clrf DCF_MONAT
	clrf DCF_TAG
	clrf DCF_WTAG
	clrf ZZ_UZ
	clrf SEK_UZ
	clrf MIN_UZ
	clrf STD_UZ
	movlw d'1'				;laden der Register mit Wert 1
	movwf WTAG_UZ
	movwf TAG_UZ
	movwf MONAT_UZ
	clrf JAHR_UZ			;Lschen der Register
	clrf SEK_SZ
	clrf MIN_SZ
	clrf STD_SZ
							;Laden der min/max Toleranzen fr die Bitlnge des DCF Signals
	movlw 0x15				;EEPROM Adresse
	call readEE				;
	movwf DCF_L_MIN			;Low Bit min Wert
	movlw 0x16				;EEPROM Adresse
	call readEE				;
	movwf DCF_L_MAX			;Low Bit max Wert
	movlw 0x17				;EEPROM Adresse
	call readEE				;
	movwf DCF_H_MIN			;High Bit min Wert
	movlw 0x18				;EEPROM Adresse
	call readEE				;
	movwf DCF_H_MAX			;High Bit max Wert

	movlw 0xFF
	movwf DISP_NEU
	movlw d'1'				;lade Register mit Wert 1
	movwf MENUE_FF			;
	call menue_stat			;Aufruf des Unterprogramms, Rcksetzen von MENUE_FF und laden der Startwerte
	bsf PCLATH,3			;Select Page 1
	call Init_display		;Initialisierung des Displays
	bcf PCLATH,3			;Select Page 0

	movlw b'10100000'		;Enable Interupts, TMR0 Interupt
	movwf INTCON


	movlw 0x14				;EEPROM Adresse fr DCF Updatezeit, Bit 7: 0 = auto, 1 = manuell
	call readEE				;
	movwf TEMP_13			;Kopieren der Daten des W-Registers ins temporre F-Register
	btfsc TEMP_13,7			;prfen, ob Bit fr manuelles Update gesetzt ist	
	goto main				;Bit ist gesetzt, weiter zur Hauptprogrammschleife
warte_20					;Bit ist nicht gesetzt => Wartezeit von 20 sek bevor Update Routine gestartet wird
	bsf PCLATH,3			;Select Page 1
	btfsc DISP_NEU,0		;Prfen ob Bit fr Aktualisierung der Anzeige nicht gesetzt ist
	call disp_uhr			;
	bcf PCLATH,3			;Select Page 0	
	movlw d'20'
	subwf SEK_UZ,w
	btfss STATUS,Z
	goto warte_20			;
	bsf DCF_MESS_STAT,0		;20 sek sind um, setze das Update Bit
	
;Testbereich Anfang

;Testbereich Ende

main						;Beginn der Hauptschleife
	call tast_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,0			;prfen ob Bit fr Befehl Menue low ist
	call menue_stat			;wenn ja berspringe diesen Befehl, wenn nein Aufruf des Unterprogramms fr Menue Flip Flop
	btfsc MENUE_FF,0			
	goto menue_1			;springe zum Programmteil "Menue"

	btfss BEFEHL,1			;Prfen ob OK Taste fr Umschaltung Anzeige Sternzeit/datum gedrckt ist
	goto abfrage_disp_stat	;
	bsf DISP_NEU,1
	bsf DISP_NEU,2
	btfss DISP_NEU,7
	goto $+3
	bcf DISP_NEU,7			;
	goto $+2
	bsf DISP_NEU,7			;
	movlw d'250'			;Wartezeit von 250ms
	movwf LOOP_1			;
	call wait 		
abfrage_disp_stat
	bsf PCLATH,3			;Select Page 1
	btfsc DISP_NEU,0		;Prfen ob bit fr aktualisierung der Anzeige nicht gesetzt ist
	call disp_uhr			;
	bcf PCLATH,3			;Select Page 0

	btfss DISP_NEU,7		;Prfen ob bit fr Anzeige Sternzeit gesetzt ist
	goto $+6
	bsf PCLATH,3			;Select Page 1
	btfsc DISP_NEU,1		;Prfen ob bit fr aktualisierung der Anzeige Sternzeit nicht gesetzt ist
	call disp_sz
	bcf PCLATH,3			;Select Page 0
	goto $+5				;
	bsf PCLATH,3			;Select Page 1	
	btfsc DISP_NEU,2		;Prfen ob bit fr aktualisierung der Anzeige Datum nicht gesetzt ist
	call disp_datum
	bcf PCLATH,3			;Select Page 0
	btfss DCF_MESS_STAT,0	
	goto main

update_uhr					;Programmteil zur Auswertung der DCF Signale
	bsf PCLATH,3			;Select Page 1	
	movlw 0x8D				;Zeichen 14 in der 1. Zeile (=0x80 + 0x00), Zeichen 14-16 werden gelscht
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw 0x20				;blank
	call disp_daten	;bertrage das Zeichen
	movlw 0x20				;blank
	call disp_daten	;bertrage das Zeichen
	movlw 0x20				;blank
	call disp_daten	;bertrage das Zeichen
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
	clrf TBIT_H				;lsche Register
	clrf TBIT_L
	clrf TBIT_H_TEMP
	clrf TBIT_L_TEMP	
	movlw 0x04				;lade Vorteiler mit Wert 4
	movwf TBIT_C
	clrf DCF_BIT
	clrf DCF_MESS_STAT		;lsche Status Register und setze Bit 0
	bsf DCF_MESS_STAT,0
startbit_prfen
	btfss DCF_MESS_STAT,4
	goto startbit_prfen1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT		
	movlw d'60'
	subwf DCF_BIT,w
	btfsc STATUS,Z			;
	goto fehler
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
startbit_prfen1
	btfss DCF_MESS_STAT,3	;wenn statusbit fr neues Datentelegramm = high, berspringe nchsten Befehl 
	goto startbit_prfen
	clrf DCF_BIT			;
;dcfbit0					;Startbit: ist immer 0
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfsc DCF_BIT_TEMP,0
	goto fehler
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
dcfbit1_16					;Allgemeine Infos, werden nicht ausgewertet.
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
	movlw d'16'
	subwf DCF_BIT,w
	btfss STATUS,Z
	goto dcfbit1_16
;dcfbit17					;bit 17+18: MEZ/MESZ
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	clrf DCF_MESZ
	btfss DCF_BIT_TEMP,0
	bcf DCF_MESZ,1
	btfsc DCF_BIT_TEMP,0
	bsf DCF_MESZ,1
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit18					;bit 17+18: MEZ/MESZ
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_MESZ,0
	btfsc DCF_BIT_TEMP,0
	bsf DCF_MESZ,0
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
dcfbit19					;bit 19: Schaltsekunde in den nchsten 59min, wird nicht ausgewertet
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit20					;Startbit fr Zeitinformation, immer 1
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	goto fehler
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit21					;Minuten Einer: Wertigkeit 1
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	clrf DCF_MIN
	clrf DCF_PRUEFBIT
	btfss DCF_BIT_TEMP,0
	bcf DCF_MIN,0
	btfsc DCF_BIT_TEMP,0
	bsf DCF_MIN,0
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit22					;Minuten Einer: Wertigkeit 2
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_MIN,1
	btfsc DCF_BIT_TEMP,0
	bsf DCF_MIN,1
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit23					;Minuten Einer: Wertigkeit 4
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_MIN,2
	btfsc DCF_BIT_TEMP,0
	bsf DCF_MIN,2
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit24					;Minuten Einer: Wertigkeit 8
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_MIN,3
	btfsc DCF_BIT_TEMP,0
	bsf DCF_MIN,3
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit25					;Minuten Zehner: Wertigkeit 1
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	clrf TEMP_11
	btfss DCF_BIT_TEMP,0
	bcf TEMP_11,0
	btfsc DCF_BIT_TEMP,0
	bsf TEMP_11,0
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit26					;Minuten Zehner: Wertigkeit 2
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf TEMP_11,1
	btfsc DCF_BIT_TEMP,0
	bsf TEMP_11,1
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit27					;Minuten Zehner: Wertigkeit 4
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf TEMP_11,2
	btfsc DCF_BIT_TEMP,0
	bsf TEMP_11,2
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
	movfw TEMP_11			;Kombinieren des 10er und 1er Wertes in eine Zahl
	movwf TEMP_13			;
	movfw DCF_MIN			;
	movwf TEMP_14			;
	call convert_2dig		;Aufruf des Unterprogramms
	movfw TEMP_15			;Rcksichern des kombinierten Wertes im DCF77 Minuten Register
	movwf DCF_MIN
;dcfbit28					;Prfbit
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	btfsc DCF_PRUEFBIT,0	;wenn Zhler gerade, weiter im Programm, wenn ungerade: Fehler und Abbruch
	goto fehler
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit29					;Stunden Einer: Wertigkeit 1
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	clrf DCF_STD
	clrf DCF_PRUEFBIT
	btfss DCF_BIT_TEMP,0
	bcf DCF_STD,0
	btfsc DCF_BIT_TEMP,0
	bsf DCF_STD,0
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit30					;Stunden Einer: Wertigkeit 2
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_STD,1
	btfsc DCF_BIT_TEMP,0
	bsf DCF_STD,1
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit31					;Stunden Einer: Wertigkeit 4
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_STD,2
	btfsc DCF_BIT_TEMP,0
	bsf DCF_STD,2
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit32					;Stunden Einer: Wertigkeit 8
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_STD,3
	btfsc DCF_BIT_TEMP,0
	bsf DCF_STD,3
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit33					;Stunden Zehner: Wertigkeit 1
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	clrf TEMP_11
	btfss DCF_BIT_TEMP,0
	bcf TEMP_11,0
	btfsc DCF_BIT_TEMP,0
	bsf TEMP_11,0
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit34					;Stunden Zehner: Wertigkeit 2
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf TEMP_11,1
	btfsc DCF_BIT_TEMP,0
	bsf TEMP_11,1
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
	movfw TEMP_11			;Kombinieren des 10er und 1er Wertes in eine Zahl
	movwf TEMP_13			;
	movfw DCF_STD			;
	movwf TEMP_14			;
	call convert_2dig		;Aufruf des Unterprogramms
	movfw TEMP_15			;Rcksichern des kombinierten Wertes im DCF77 Minuten Register
	movwf DCF_STD
;dcfbit35					;Prfbit
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	btfsc DCF_PRUEFBIT,0	;wenn Zhler gerade, weiter im Programm, wenn ungerade: Fehler und Abbruch
	goto fehler
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit36					;Tag Einer: Wertigkeit 1
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	clrf DCF_TAG
	clrf DCF_PRUEFBIT
	btfss DCF_BIT_TEMP,0
	bcf DCF_TAG,0
	btfsc DCF_BIT_TEMP,0
	bsf DCF_TAG,0
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit37					;Tag Einer: Wertigkeit 2
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_TAG,1
	btfsc DCF_BIT_TEMP,0
	bsf DCF_TAG,1
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit38					;Tag Einer: Wertigkeit 4
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_TAG,2
	btfsc DCF_BIT_TEMP,0
	bsf DCF_TAG,2
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit39					;Tag Einer: Wertigkeit 8
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_TAG,3
	btfsc DCF_BIT_TEMP,0
	bsf DCF_TAG,3
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit40					;Tag Zehner: Wertigkeit 1
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	clrf TEMP_11
	btfss DCF_BIT_TEMP,0
	bcf TEMP_11,0
	btfsc DCF_BIT_TEMP,0
	bsf TEMP_11,0
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit41					;Tag Zehner: Wertigkeit 2
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf TEMP_11,1
	btfsc DCF_BIT_TEMP,0
	bsf TEMP_11,1
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
	movfw TEMP_11			;Kombinieren des 10er und 1er Wertes in eine Zahl
	movwf TEMP_13			;
	movfw DCF_TAG			;
	movwf TEMP_14			;
	call convert_2dig		;Aufruf des Unterprogramms
	movfw TEMP_15			;Rcksichern des kombinierten Wertes im DCF77 Minuten Register
	movwf DCF_TAG
;dcfbit42					;Wochentag Einer: Wertigkeit 1
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	clrf DCF_WTAG
	btfss DCF_BIT_TEMP,0
	bcf DCF_WTAG,0
	btfsc DCF_BIT_TEMP,0
	bsf DCF_WTAG,0
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit43					;Wochentag Einer: Wertigkeit 2
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_WTAG,1
	btfsc DCF_BIT_TEMP,0
	bsf DCF_WTAG,1
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit44					;Wochentag Einer: Wertigkeit 4
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_WTAG,2
	btfsc DCF_BIT_TEMP,0
	bsf DCF_WTAG,2
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit45					;Monat Einer: Wertigkeit 1
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	clrf DCF_MONAT
	btfss DCF_BIT_TEMP,0
	bcf DCF_MONAT,0
	btfsc DCF_BIT_TEMP,0
	bsf DCF_MONAT,0
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit46					;Monat Einer: Wertigkeit 2
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_MONAT,1
	btfsc DCF_BIT_TEMP,0
	bsf DCF_MONAT,1
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit47					;Monat Einer: Wertigkeit 4
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_MONAT,2
	btfsc DCF_BIT_TEMP,0
	bsf DCF_MONAT,2
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit48					;Monat Einer: Wertigkeit 8
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_MONAT,3
	btfsc DCF_BIT_TEMP,0
	bsf DCF_MONAT,3
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit49					;Monat Zehner: Wertigkeit 1
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	clrf TEMP_11
	btfss DCF_BIT_TEMP,0
	bcf TEMP_11,0
	btfsc DCF_BIT_TEMP,0
	bsf TEMP_11,0
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
	movfw TEMP_11			;Kombinieren des 10er und 1er Wertes in eine Zahl
	movwf TEMP_13			;
	movfw DCF_MONAT			;
	movwf TEMP_14			;
	call convert_2dig		;Aufruf des Unterprogramms
	movfw TEMP_15			;Rcksichern des kombinierten Wertes im DCF77 Minuten Register
	movwf DCF_MONAT
;dcfbit50					;Jahr Einer: Wertigkeit 1
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	clrf DCF_JAHR
	btfss DCF_BIT_TEMP,0
	bcf DCF_JAHR,0
	btfsc DCF_BIT_TEMP,0
	bsf DCF_JAHR,0
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit51					;Jahr Einer: Wertigkeit 2
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_JAHR,1
	btfsc DCF_BIT_TEMP,0
	bsf DCF_JAHR,1
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit52					;Jahr Einer: Wertigkeit 4
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_JAHR,2
	btfsc DCF_BIT_TEMP,0
	bsf DCF_JAHR,2
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit53					;Jahr Einer: Wertigkeit 8
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf DCF_JAHR,3
	btfsc DCF_BIT_TEMP,0
	bsf DCF_JAHR,3
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit54					;Jahr Zehner: Wertigkeit 1
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	clrf TEMP_11
	btfss DCF_BIT_TEMP,0
	bcf TEMP_11,0
	btfsc DCF_BIT_TEMP,0
	bsf TEMP_11,0
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit55					;Jahr Zehner: Wertigkeit 2
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf TEMP_11,1
	btfsc DCF_BIT_TEMP,0
	bsf TEMP_11,1
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit56					;Jahr Zehner: Wertigkeit 4
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf TEMP_11,2
	btfsc DCF_BIT_TEMP,0
	bsf TEMP_11,2
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit57					;Jahr Zehner: Wertigkeit 8
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfss DCF_BIT_TEMP,0
	bcf TEMP_11,3
	btfsc DCF_BIT_TEMP,0
	bsf TEMP_11,3
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
	movfw TEMP_11			;Kombinieren des 10er und 1er Wertes in eine Zahl
	movwf TEMP_13			;
	movfw DCF_JAHR			;
	movwf TEMP_14			;
	call convert_2dig		;Aufruf des Unterprogramms
	movfw TEMP_15			;Rcksichern des kombinierten Wertes im DCF77 Minuten Register
	movwf DCF_JAHR
;dcfbit58					;Prfbit
	btfss DCF_MESS_STAT,4
	goto $-1
	bcf DCF_MESS_STAT,4
	incf DCF_BIT	
	btfsc DCF_MESS_STAT,1
	goto fehler
	btfsc DCF_BIT_TEMP,0	;wenn 1 erhhe den Prfbitzhler um 1
	incf DCF_PRUEFBIT
	btfsc DCF_PRUEFBIT,0	;wenn Zhler gerade, weiter im Programm, wenn ungerade: Fehler und Abbruch
	goto fehler
	bsf PCLATH,3			;Select Page 1	
	call disp_zahl
	bcf PCLATH,3			;Select Page 0
;dcfbit59					;keine Absenkung, mit der nchsten negativen Flanke beginnt die neue Minute
	bcf DCF_MESS_STAT,3		;lschen des Statusbit	
	bcf DCF_MESS_STAT,4
	btfss DCF_MESS_STAT,3	;wenn statusbit fr neuen datensatz = high, berspringe nchsten Befehl 
	goto $-2

	btfsc DCF_MESS_STAT,2	;warten auf die negative Flanke des DCF signals
	goto $-1
	movlw HIGH_BYTE_S		;Empfang des DCF Signals war Fehlerfrei, rcksetzen der Zhler fr den sekundentakt der internen Uhr
	movwf COUNT_H_S
	movlw LOW_BYTE_S
	movwf COUNT_L_S
	movlw 0x04
	movwf COUNT_N	
	clrf DCF_BIT
	movlw d'10'				;Wartezeit von 10ms, um sicherzustellen, da sekunde nicht zustzlich erhht wird
	movwf LOOP_1			;
	call wait 		
	clrf SEK_UZ

	clrf DCF_MESS_STAT		;lschen der Register
	clrf DCF_COUNT
	movfw DCF_STD			;Kopieren der Temporren Register in die aktiven Register
	movwf STD_UZ
	movfw DCF_MIN
	movwf MIN_UZ
	movfw DCF_TAG
	movwf TAG_UZ
	movfw DCF_MONAT
	movwf MONAT_UZ
	movfw DCF_JAHR
	movwf JAHR_UZ
	movfw DCF_MESZ
	movwf ZZ_UZ
	movfw DCF_WTAG
	movwf WTAG_UZ

	bsf PCLATH,3			;Select Page 1
	call clear_display
	bcf PCLATH,3			;Select Page 0
	bsf PCLATH,4			;Select Page 2
	call sternzeit			;Aufruf des Unterprogramms zur Sternzeitberechnung
	bcf PCLATH,4			;Select Page 0
	bsf DISP_NEU,2			;setze das Statusbit zur Aktualisierung der Sternzeit im Display
	goto main

fehler
	incf DCF_COUNT			;Zhler wird erhht bei jedem Fehlerhaft empfangenen Telegramm
	clrf DCF_MESS_STAT
	bsf DCF_MESS_STAT,0		;
	movlw d'5'				;nach 5 Fehlversuchen wird die Update Routine beendet
	subwf DCF_COUNT,w
	btfss STATUS,Z
	goto update_uhr
	clrf DCF_MESS_STAT	
	clrf DCF_COUNT
	bsf PCLATH,3			;Select Page 1
	call clear_display
	bcf PCLATH,3			;Select Page 0
	bsf DISP_NEU,2
	goto main


menue_1						;Programmverzweigung fr Mensteuerung
	bsf PCLATH,3			;Select Page 1
	btfsc DISP_NEU,4
	call disp_menue_1
	bcf PCLATH,3			;Select Page 0
	btfsc BEFEHL,5			;Prfen ob Pfeiltaste "down" nicht gedrckt ist (low ist)
	goto menue_1dn			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,4			;Prfen ob Pfeiltaste "up" nicht gedrckt ist (low ist)
	goto menue_1up			;wenn ja berspringe diesen Befehl

	btfsc BEFEHL,3			;Prfen ob "Right" Taste nicht gedrckt ist (low ist)
	call inc_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;Prfen ob "LEFT" Taste nicht gedrckt ist (low ist)
	call dec_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc TEMP_DIG_STAT,0	;prfen ob bit in Register gesetzt ist
	call dig_temp_kat		;wenn ja berspringe diesen Befehl

	btfss BEFEHL,1			;Prfen ob "OK" Taste gedrckt ist (high ist)
	goto main				;wenn ja berspringe diesen Befehl
	movfw MENUE_COUNT
	movwf MENUE_1_CNT

	bsf PCLATH,3			;Select Page 1
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	call cursor_on			;Unterprogramm zum einschalten des Cursors
	bcf PCLATH,3			;Select Page 0

	movlw d'1'				;
	subwf MENUE_1_CNT, w	;
	btfsc STATUS, Z		
	goto menu_dcfupdate_set	; Aufruf des Programmteils menu_dcfupdate_set
	movlw d'2'				;
	subwf MENUE_1_CNT, w	;
	btfsc STATUS, Z		
	goto menu_standort_set	; Aufruf des Programmteils menu_standort_set
	movlw d'3'				;
	subwf MENUE_1_CNT, w	;
	btfsc STATUS, Z		
	goto menu_datum_set		; Aufruf des Programmteils menu_datum_set
	movlw d'4'				;
	subwf MENUE_1_CNT, w	;
	btfsc STATUS, Z		
	goto menu_parameter_set	; Aufruf des Programmteils menu_parameter_set
	movlw d'5'				;
	subwf MENUE_1_CNT, w	;
	btfsc STATUS, Z		
	goto menu_q1hz_set		; Aufruf des Programmteils menu_q1hz_set
	goto main

menu_datum_set
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movlw d'9'				;Anzeige hat 9 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	movfw JAHR_UZ			;kopiere aktuelle Werte in Temp Register
	movwf DCF_JAHR			;
	movfw MONAT_UZ			;
	movwf DCF_MONAT			;
	movfw TAG_UZ			;
	movwf DCF_TAG			;
	movfw WTAG_UZ			;
	movwf DCF_WTAG			;	
	movfw STD_UZ			;
	movwf DCF_STD
	movfw MIN_UZ
	movwf DCF_MIN
	movfw SEK_UZ
	movwf SEK_TEMP
	btfss ZZ_UZ,7			;prfen ob Bit 7 im Register gesetzt ist: 1=-, 0=+ 
	goto $+3
	movlw A'-'				;Bit ist gesetzt: = -
	goto $+2
	movlw A'+'				;Bit ist nicht gesetzt: = +
	movwf PM_TEM
	movfw ZZ_UZ
	movwf DCF_MESZ
	bcf DCF_MESZ,7
	bsf TEMP_DIG_STAT,0		;setze Statusbit
	bsf PCLATH,3			;Select Page 1
	call disp_set_datum
	call dig_datum1			;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0
loop_datum_set
	call tast_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,0			;prfen ob Bit fr Befehl Menue low ist
	goto copy_datum1		;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,1			;Prfen ob "OK" Taste nicht gedrckt ist (low ist)
	goto copy_datum			;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,3			;Prfen ob "Right" Taste nicht gedrckt ist (low ist)
	call inc_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;Prfen ob "LEFT" Taste nicht gedrckt ist (low ist)
	call dec_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	bsf PCLATH,3			;Select Page 1
	btfsc TEMP_DIG_STAT,0	;prfen ob bit in register nicht gesetzt ist
	call dig_datum1			;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0
	btfss BEFEHL,4			;Prfen ob "UP" Taste gedrckt ist (low ist)
	goto $+4
	bsf TEMP_DIG_STAT,1		;
	bsf PCLATH,3			;Select Page 1
	call dig_datum1			;wenn ja berspringe diesen Befehl
	bsf PCLATH,3			;Select Page 1
	btfsc BEFEHL,5			;Prfen ob "DOWN" Taste nicht gedrckt ist (low ist)
	call dig_datum1			;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0
	goto loop_datum_set		;zurck zur Schleife 

copy_datum					;Kopiere Temp Register in Datum/Uhrzeit Register
	movlw 0x10				;Laden der Konstanten fr den Uhrzeittakt, damit Zhler mit voller Sekunde beginnt
	movwf COUNT_H_S		;
	movlw 0x42
	movwf COUNT_L_S
	movlw 0x02				;Setzen des Zhlers, Digit:= 2	
	movwf COUNT_N			;
	movfw DCF_JAHR			;kopieren der temporren Register in die Datum/Uhrzeit/Zeitzonen Register
	movwf JAHR_UZ
	movfw DCF_MONAT
	movwf MONAT_UZ
	movfw DCF_TAG
	movwf TAG_UZ
	movfw DCF_WTAG
	movwf WTAG_UZ
	movfw DCF_STD
	movwf STD_UZ
	movfw DCF_MIN
	movwf MIN_UZ
	movfw SEK_TEMP
	movwf SEK_UZ
	movlw A'+'
	subwf PM_TEM,w
	btfss STATUS,Z
	bsf DCF_MESZ,7
	movfw DCF_MESZ
	movwf ZZ_UZ
	bsf PCLATH,4			;Select Page 2
	call sternzeit
	bcf PCLATH,4			;Select Page 0
	bsf DISP_NEU,2
copy_datum1
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	call menue_stat			;
	goto main

menu_standort_set
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movlw d'4'				;Anzeige hat 4 Stellen, die gesetzt werden, E/W, Grad, Min, Sek
	movwf TEMP_CNT_MAX		;
	call read_position		;lade die Positionsdaten aus dem internen EEPROM in die F-Register
	bsf TEMP_DIG_STAT,0		;setze Statusbit
	bsf PCLATH,3			;Select Page 1
	call disp_set_standort
	call dig_standort1		;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0
loop_standort_set
	call tast_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,0			;prfen ob Bit fr Befehl Menue low ist
	goto copy_standort1		;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,1			;Prfen ob "OK" Taste nicht gedrckt ist (low ist)
	goto copy_standort		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,3			;Prfen ob "Right" Taste nicht gedrckt ist (low ist)
	call inc_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;Prfen ob "LEFT" Taste nicht gedrckt ist (low ist)
	call dec_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	bsf PCLATH,3			;Select Page 1
	btfsc TEMP_DIG_STAT,0	;prfen ob bit in register nicht gesetzt ist
	call dig_standort1		;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0
	btfss BEFEHL,4			;Prfen ob "UP" Taste gedrckt ist (low ist)
	goto $+4				;
	bsf TEMP_DIG_STAT,1		;
	bsf PCLATH,3			;Select Page 1
	call dig_standort1		;wenn ja berspringe diesen Befehl
	bsf PCLATH,3			;Select Page 1
	btfsc BEFEHL,5			;Prfen ob "DOWN" Taste nicht gedrckt ist (low ist)
	call dig_standort1		;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0
	goto loop_standort_set	;zurck zur Schleife 

copy_standort				;Speichern der Temp Register im internen EEPROM
	movfw PM_TEM			;kopieren der temporren Register ins interne EEPROM, Ost=0, West=1
	movwf TEMP_13			;
	movlw 0x10
	call writeEE			;
	movfw GRD_TEM			;kopieren der temporren Register ins interne EEPROM, Winkelgrad
	movwf TEMP_13			;
	movlw 0x11
	call writeEE			;
	movfw MIN_TEM			;kopieren der temporren Register ins interne EEPROM, Winkelminuten
	movwf TEMP_13			;
	movlw 0x12
	call writeEE			;
	movfw MIN_TEM_01		;kopieren der temporren Register ins interne EEPROM, Winkelsekunden
	movwf TEMP_13			;
	movlw 0x13
	call writeEE			;

	movlw d'250'			;Wartezeit von 250ms, damit der Schreibvorgang vor
	movwf LOOP_1			;  dem nchsten Lesevorgang beendet werden kann
	call wait 			

	movfw JAHR_UZ			;kopiere Datum und Uhrzeitwerte in Temp Register (fr Sternzeitberechnung)
	movwf DCF_JAHR			;
	movfw MONAT_UZ			;
	movwf DCF_MONAT			;
	movfw TAG_UZ			;
	movwf DCF_TAG			;
	movfw ZZ_UZ				;
	movwf DCF_MESZ
	movfw STD_UZ			;
	movwf DCF_STD
	movfw MIN_UZ
	movwf DCF_MIN

	bsf PCLATH,4			;Select Page 2
	call sternzeit
	bcf PCLATH,4			;Select Page 0
copy_standort1
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	call menue_stat			;
	goto main

menu_dcfupdate_set			;Programmteil zum setzen der DCF Update Einstellungen
	movlw 0x14				;Adresse von der aus dem EEProm gelesen werden soll
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf TEMP_13			;Kopieren der Daten des W-Registers ins F-Register
	movlw d'1'				;prfen ob Zhler auf 1 steht (manuelles update)
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z
	goto menu_dcfup_aut		;Zhler ist nicht 1, gehe zum automatische update
	bsf TEMP_13,7			;setze bit fr manuelles update und schreibe Wert zurck ins EEPROM
	movlw 0x14				;Adresse EEProm
	call writeEE			;
	bsf PCLATH,3			;Select Page 1
	call cursor_off			;Cursor ausschalten
	call disp_set_dcfup_m
	bcf PCLATH,3			;Select Page 0
	movlw d'250'			;Wartezeit von 250ms
	movwf LOOP_1			;
	call wait 			
loop_dcfup_man
	call tast_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,0			;prfen ob Bit fr Befehl Menue low ist
	goto copy_dcfup_man1	;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,1			;Prfen ob "OK" Taste nicht gedrckt ist (low ist)
	goto copy_dcfup_man		;wenn ja berspringe diesen Befehl
	goto loop_dcfup_man		;zurck zur Schleife
copy_dcfup_man
	bsf DCF_MESS_STAT,0		;setze Statusbit
copy_dcfup_man1
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	call menue_stat			;
	goto main

menu_dcfup_aut				;Automatisches DCF Update
	bcf TEMP_13,7			;lsche Bit fr manuelles Update
	movfw TEMP_13			;
	movwf PM_TEM			;
	bsf PCLATH,3			;Select Page 1
	call disp_set_dcfup_a
	bcf PCLATH,3			;Select Page 0
	movlw d'250'			;Wartezeit von 250ms
	movwf LOOP_1			;
	call wait 	
loop_dcfup_aut
	call tast_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,0			;prfen ob Bit fr Befehl Menue low ist
	goto copy_dcfup_aut1	;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,1			;Prfen ob "OK" Taste nicht gedrckt ist (low ist)
	goto copy_dcfup_aut		;wenn ja berspringe diesen Befehl
	btfss BEFEHL,4			;Prfen ob "UP" Taste gedrckt ist (low ist)
	goto $+4				;
	bsf TEMP_DIG_STAT,1		;
	bsf PCLATH,3			;Select Page 1
	call dig_dcfup_aut		;wenn ja berspringe diesen Befehl
	bsf PCLATH,3			;Select Page 1
	btfsc BEFEHL,5			;Prfen ob "DOWN" Taste nicht gedrckt ist (low ist)
	call dig_dcfup_aut		;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0
	goto loop_dcfup_aut		;zurck zur Schleife 
copy_dcfup_aut
	movfw PM_TEM			;Speichern des Wertes im EEPROM
	movwf TEMP_13
	movlw 0x14				;EEPROM Adresse
	call writeEE			;
copy_dcfup_aut1
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	call menue_stat			;
	goto main

menu_parameter_set			;Programmteil zum Setzen der Toleranzen (min/max Werte) fr die Bitlnge des DCF Signals
	movlw d'1'				;prfen ob Zhler auf 1 steht (manuelles update)
	subwf TEMP_DIG_CNT,w
	btfss STATUS,Z
	goto menu_para_set_aut	;Zhler ist nicht 1, gehe zum automatische update

	movlw d'1'				;manuelles Update, Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	movlw d'4'				;Anzeige hat 4 Stellen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	movfw DCF_L_MIN			;Low Bit min Wert
	movwf MIN_TEM_01		;
	movfw DCF_L_MAX			;Low Bit max Wert
	movwf PM_TEM			;
	movfw DCF_H_MIN			;High Bit min Wert
	movwf STD_TEMP			;
	movfw DCF_H_MAX			;High Bit max Wert
	movwf REST_SEK_TEMP		;
	bsf TEMP_DIG_STAT,0		;setze Statusbit
	bsf PCLATH,3			;Select Page 1
	call disp_set_para3		;Aufruf des Unterprogramms zur Anzeige der min/max Werte
	call dig_para_man1		;
	bcf PCLATH,3			;Select Page 0
loop_para_man
	call tast_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,0			;prfen ob Bit fr Befehl Menue low ist
	goto copy_parameter1	;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife	
	btfsc BEFEHL,1			;Prfen ob "OK" Taste nicht gedrckt ist (low ist)
	goto copy_parameter		;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,3			;Prfen ob "Right" Taste nicht gedrckt ist (low ist)
	call inc_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	btfsc BEFEHL,2			;Prfen ob "LEFT" Taste nicht gedrckt ist (low ist)
	call dec_dig_temp_cnt	;wenn ja berspringe diesen Befehl
	bsf PCLATH,3			;Select Page 1
	btfsc TEMP_DIG_STAT,0	;prfen ob bit in register nicht gesetzt ist
	call dig_para_man1		;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0
	btfss BEFEHL,4			;Prfen ob "UP" Taste gedrckt ist (low ist)
	goto $+4				;
	bsf TEMP_DIG_STAT,1		;
	bsf PCLATH,3			;Select Page 1
	call dig_para_man1		;wenn ja berspringe diesen Befehl
	bsf PCLATH,3			;Select Page 1
	btfsc BEFEHL,5			;Prfen ob "DOWN" Taste nicht gedrckt ist (low ist)
	call dig_para_man1		;wenn ja berspringe diesen Befehl
	bcf PCLATH,3			;Select Page 0
	goto loop_para_man		;zurck zur Schleife 
	

menu_para_set_aut
	bsf PCLATH,3			;Select Page 1
	call cursor_off
	call disp_set_para1
	bcf PCLATH,3			;Select Page 0
	movlw d'250'			;Wartezeit von 250ms
	movwf LOOP_1			;
	call wait 	
	clrf MIN_TEM_01			;= DCF LBit: Summe der DCF L-Bits, L-Register
	clrf MIN_TEM			;= DCF LBit: Summe der DCF L-Bits, H-Register
	clrf GRD_TEM			;= DCF LBit: Anzahl der DCF L-Bits
	clrf PM_TEM				;= DCF LBit: Mittelwert der DCF L-Bits
	clrf STD_TEMP			;= DCF HBit: Summe der DCF H-Bits, L-Register
	clrf MIN_TEMP			;= DCF HBit: Summe der DCF H-Bits, H-Register
	clrf SEK_TEMP			;= DCF HBit: Anzahl der DCF H-Bits
	clrf REST_SEK_TEMP		;= DCF HBit: Mittelwert der DCF H-Bits
	movlw d'150'			;Startwert zur Unterscheidung H/L-Bit
	movwf DCF_PRUEFBIT
	clrf TBIT_H
	clrf TBIT_L
	movlw 0x04
	movwf TBIT_C
	clrf DCF_BIT
	clrf DCF_MESS_STAT
	bsf DCF_MESS_STAT,0
dcf_bitlaenge
	btfsc DCF_MESS_STAT,4
	call para_routine
	call tast_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,0			;prfen ob Bit fr Befehl Menue low ist
	goto copy_parameter1	;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife
	btfss DCF_MESS_STAT,5
	goto dcf_bitlaenge
	btfss DCF_MESS_STAT,6
	goto dcf_bitlaenge
	call dcf_minmax
menu_para_set1
	call tast_befehl		;Aufruf Unterprogramm zum Laden der Tastaturbefehle
	btfsc BEFEHL,0			;prfen ob Bit fr Befehl Menue low ist
	goto copy_parameter1	;wenn ja berspringe diesen Befehl, wenn nein Rcksprung in die Hauptschleife
	btfsc BEFEHL,1			;Prfen ob "OK" Taste nicht gedrckt ist (low ist)
	goto copy_parameter		;wenn ja berspringe diesen Befehl
	goto menu_para_set1

copy_parameter				;Kopieren der Temp Register in die Arbeitsregister und Speichern der Werte im EEPROM
	movfw MIN_TEM_01		;Low Bit min Wert
	movwf DCF_L_MIN			;
	movwf TEMP_13			;
	movlw 0x15				;EEPROM Adresse
	call writeEE			;
	movfw PM_TEM			;Low Bit max Wert
	movwf DCF_L_MAX			;
	movwf TEMP_13			;
	movlw 0x16				;EEPROM Adresse
	call writeEE			;
	movfw STD_TEMP			;High Bit min Wert
	movwf DCF_H_MIN			;
	movwf TEMP_13			;
	movlw 0x17				;EEPROM Adresse
	call writeEE			;
	movfw REST_SEK_TEMP		;High Bit max Wert
	movwf DCF_H_MAX			;
	movwf TEMP_13			;
	movlw 0x18				;EEPROM Adresse
	call writeEE			;
copy_parameter1
	clrf DCF_MESS_STAT
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	call menue_stat			;
	goto main

menu_q1hz_set				;Unterprogramm zur Anzeige der mittleren Abweichung zwischen interner Uhr und DCF Signal
							;in ppm, verursacht durch eine zu hohe bzw. zu niedrige Oszillator Frequenz.
							;Lschen der Register
	clrf TAGE_L				;Wird verwendet fr Sekunden Zhler L Byte
	clrf TAGE_H				;Wird verwendet fr Sekunden Zhler H Byte
	clrf TEMP_11			;Wird verwendet fr Zhler Abweichung L Byte
	clrf TEMP_12			;Wird verwendet fr Zhler Abweichung M Byte
	clrf TEMP_13			;Wird verwendet fr Zhler Abweichung H Byte
	clrf STD_TEMP			;wird fr Zwischenspeicherung Abweichung 1000er/100er verwendet
	clrf MIN_TEMP			;wird fr Zwischenspeicherung Abweichung 10er/1er verwendet
	clrf SEK_TEMP			;wird fr Zwischenspeicherung Sekunden 1000er/100er verwendet
	clrf REST_SEK_TEMP		;wird fr Zwischenspeicherung Sekunden 10er/1er verwendet
	movlw A' '				;ASCII: Blank
	movwf JAHR_4			;JAHR_4 wird verwendet fr Vorzeichen Abweichung

	bsf PCLATH,3			;Select Page 1
	call clear_display		;Aufruf des Unterprogramms zum lschen des Display
	call cursor_off			;Aufruf des Unterprogramms zum ausschalten des Cursors
	call disp_q1hz			;Aufruf des Unterprogramms zur Anzeige auf dem Display
	bcf PCLATH,3			;Select Page 0

	clrf MESS_END			;Lschen des Status Registers
	btfss IN_TASTER			;Abfrage des Men Tasters
	goto menu_q1hz_end		;Taster ist gedrckt => springe zum Ende der Programmschleife
	btfss IN_DCF			;Warten auf die positive Flanke des DCF Signals
	goto $-3				;
	btfss IN_DCF			;Warten auf die negative Flanke des DCF Signals => Beginn einer neuen Sekunde.
	bsf MESS_END,1
	btfss MESS_END,1
	goto $-3
	movlw d'4'				;Mit dieser negativen Flanke des DCF Signals wird die interne Uhr syncronisiert. 
	movwf TMR0				;(4 Microsekunden Adjustment durch Zeitdifferenz) 
	movlw HIGH_BYTE_S		;Rcksetzen des Sekundenzhlers der internen Uhr
	movwf COUNT_H_S
	movlw LOW_BYTE_S
	movwf COUNT_L_S
	movlw 0x04
	movwf COUNT_N
	clrf MESS_END			;Lsche Status Bit Register
q1hz_loop
	btfss IN_TASTER			;Abfrage des Men Tasters
	goto menu_q1hz_end		;Taster ist gedrckt => springe zum Ende der Programmschleife
	clrf TEMP_11			;lsche 24 Bit Zhler Register fr die Abweichung
	clrf TEMP_12
	clrf TEMP_13
	btfsc IN_DCF			;Warten auf die positive Flanke des DCF Signals
	bsf MESS_END,1			;Eingang des DCF Signals ist High, setzen des Status Bits fr Eingang High
	btfss MESS_END,1		;Prfen ob Eingang High ist
	goto $+3				;DCF Signal ist noch Low, weiter bei Prfung der internen Uhr  
	btfss IN_DCF			;Warten auf die negative Flanke des DCF Signals
	goto ext_first			;Negative Flanke liegt an => d.h. das DCF Signal ist "schneller" als die interne Uhr
	btfsc MESS_END,0		;Prft ob eine Sekunde der internen Uhr abgelaufen ist 
	goto int_first			;Sekunde ist abgelaufen => d.h. die interne Uhr ist "schneller" als das DCF Signal 
	goto q1hz_loop			;
int_first
	btfss IN_TASTER			;Abfrage des Men Tasters
	goto menu_q1hz_end		;Taster ist gedrckt => springe zum Ende der Programmschleife
	btfss MESS_END,0		;Prft ob das Status Bit fr volle Sekunde der internen Uhr noch gesetzt ist 
	goto $+6				;Bit ist nicht gesetzt, weiter bei Prfung des Timer 0 berlaufs
	bcf MESS_END,0			;Bit ist noch gesetzt, lsche Sekunden- und Timer 0 berlauf Bits
	bcf MESS_END,2
	incfsz TAGE_L			;erhhe das Low Byte des Sekundenzhlers um 1 und prfe auf berlauf
	goto $+2				;es liegt kein berlauf vor, 
	incf TAGE_H				;Low Byte des Sekundenzhlers ist bergelaufen, erhhe High Byte um 1
	btfss MESS_END,2		;Prft den Timer 0 berlauf
	goto $+5				;es liegt kein berlauf vor, weiter bei Prfung der negativen Flanke des DCF Signals
	bcf MESS_END,2			;Timer 0 ist bergelaufen, lsche das berlauf Bit
	incfsz TEMP_12			;Erhhe das M Byte des 24 Bit Abweichungs Registers um 1 und prfe auf berlauf
	goto $+2				;es liegt kein berlauf vor, 
	incf TEMP_13			;M Byte des 24 Bit Abweichungs Registers ist bergelaufen, erhhe H Byte um 1
	btfss IN_DCF			;prfen der negativen Flanke des DCF Signals
	goto $+6				;Negative Flanke liegt an => weiter bei sichern des korrigierten Timer 0 Wertes
	movlw d'2'				;wenn nach 0,262sek keine negative Flanke anliegt (z.B. in der 59sten Sek des DCF Protokolls),
	subwf TEMP_13,w			;  wird die Messung abgebrochen.
	btfss STATUS,Z
	goto int_first			;0,262sek Marke noch nicht erreicht, zurck zur int_first Schleife
	goto q1hz_loop			;0,262sek Marke ist erreicht, Abbruch und weiter bei nchster Messung
	movlw d'4'				;sichern des um 4 Microsekunden reduzierten (Zeitdifferenz) Timer 0 Wertes  
	subwf TMR0,w			;   im L Byte des 24 Bit Abweichungs Registers
	movwf TEMP_11
	movlw A'+'				;Vorzeichen fr Abweichung
	goto auswert_q1hz		;Weiter zur Auswertung

ext_first
	btfss IN_TASTER			;Abfrage des Men Tasters
	goto menu_q1hz_end		;Taster ist gedrckt => springe zum Ende der Programmschleife
	btfss MESS_END,1		;Prft ob das Status Bit fr positive Flanke des DCF Eingangs noch gesetzt ist 
	goto $+6				;Bit ist nicht gesetzt, weiter bei Prfung des Timer 0 berlaufs
	movlw d'4'				;Bit ist noch gesetzt, lsche DCF- und Timer 0 berlauf Bits und sichere den um 4 Microsek
	subwf TMR0,w			;  reduzierten (Zeitdifferenz) Timer 0 Wert im L Byte des 24 Bit Abweichungs Registers
	movwf TEMP_11
	bcf MESS_END,1
	bcf MESS_END,2
	btfss MESS_END,2		;Prft den Timer 0 berlauf
	goto $+5				;es liegt kein berlauf vor, weiter bei Prfung, ob eine Sekunde abgelaufen ist
	bcf MESS_END,2			;Timer 0 ist bergelaufen, lsche das berlauf Bit
	incfsz TEMP_12			;Erhhe das M Byte des 24 Bit Abweichungs Registers um 1 und prfe auf berlauf
	goto $+2				;es liegt kein berlauf vor, 
	incf TEMP_13			;M Byte des 24 Bit Abweichungs Registers ist bergelaufen, erhhe H Byte um 1
	btfss MESS_END,0		;Prft ob eine Sekunde abgelaufen ist 
	goto ext_first			;Sekunde ist noch nicht um, zurck zur ext_first Schleife
	movfw TEMP_11			;Sekunde ist um, Berechnung der Differenz des im L Byte gespeicherten und korrigierten Timer 0
	sublw d'0'				;  Wertes zum Timer 0 berlauf ( = 256 - korr Timer 0 )
	movwf TEMP_11			;Rcksichern des Ergebnisses im L Byte des 24 Bit Abweichungs Registers
	incfsz TAGE_L			;erhhe das Low Byte des Sekundenzhlers um 1 und prfe auf berlauf
	goto $+2				;es liegt kein berlauf vor, 
	incf TAGE_H				;Low Byte des Sekundenzhlers ist bergelaufen, erhhe High Byte um 1
	movlw A'-'				;Vorzeichen fr Abweichung

auswert_q1hz
	movwf JAHR_4			;Sichern des Vorzeichen fr Abweichung
	clrf MESS_END			;Lsche Status Register
	movfw TAGE_L			;kopiere die L+H Sekunden Register in die Nenner Register fr die Division
	movwf TEMP_15			;   ( Die Werte des 24 Bit Abweichungs Registers TEMP_13:TEMP_12:TEMP_11 stehen schon 
	movfw TAGE_H			;    in den Zhler Registern fr die Division)
	movwf TEMP_16
	bsf PCLATH,4			;Select Page 2 (Unterprogramm zur Division steht auf Page 2)
	call Div16				;Aufruf des Unterprogramms zur Division ( Aufgelaufene Abweichung geteilt durch Abgelaufene Sekunden)
	bcf PCLATH,4			;Select Page 0
	clrf TEMP_16			;Lsche Nenner H Byte
	movlw 0x64				;teilen durch 100 
	movwf TEMP_15			;Lade Nenner L Byte mit Wert 100
	bsf PCLATH,4			;Select Page 2
	call Div16				;Aufruf des Unterprogramms zur Division
	bcf PCLATH,4			;Select Page 0
	movfw TEMP_11
	movwf STD_TEMP			;wird fr Zwischenspeicherung Abweichung 1000er/100er verwendet
	movfw TEMP_17			;Rest L Byte = 10er/1er
	movwf MIN_TEMP			;wird fr Zwischenspeicherung Abweichung 10er/1er verwendet
	clrf TEMP_13			;Konvertieren der beiden 8 Bit Register fr die Sekunden in 2 Register 
	movfw TAGE_L
	movwf TEMP_11
	movfw TAGE_H
	movwf TEMP_12
	clrf TEMP_16			;Lsche Nenner H Byte
	movlw 0x64				;teilen durch 100 
	movwf TEMP_15			;Lade Nenner L Byte mit Wert 100
	bsf PCLATH,4			;Select Page 2
	call Div16				;Aufruf des Unterprogramms zur Division
	bcf PCLATH,4			;Select Page 0
	movfw TEMP_11
	movwf SEK_TEMP			;wird fr Zwischenspeicherung Sekunden 1000er/100er verwendet
	movfw TEMP_17			;Rest L Byte
	movwf REST_SEK_TEMP		;wird fr Zwischenspeicherung Sekunden 10er/1er verwendet

	bsf PCLATH,3			;Select Page 1
	call disp_q1hz			;Aufruf des Unterprogramms zur Anzeige auf dem Display
	bcf PCLATH,3			;Select Page 0
	goto q1hz_loop
menu_q1hz_end
	movlw d'1'				;Rcksetzen des Zhlers auf 1
	movwf TEMP_DIG_CNT
	call menue_stat			;
	goto main

menue_1up					;Programmteil zum Erhhen des Menpunktes
	bsf DISP_NEU,4			;setze das Statusbit
	incf MENUE_COUNT		;
	movlw d'6'
	subwf MENUE_COUNT, w
	btfss STATUS, Z		
	goto $+3
	movlw d'1'
	movwf MENUE_COUNT
	movlw d'1'				;lade Zhler mit Wert 1
	movwf TEMP_DIG_CNT		;
	bsf TEMP_DIG_STAT,0		;setze Statusbit fr update Cursorposition in Menpunkten Goto, Position und Speed
	movlw d'250'			;Wartezeit von 250ms
	movwf LOOP_1			;
	call wait 
	goto main

menue_1dn					;Programmteil zum Reduzieren des Menpunktes
	bsf DISP_NEU,4			;setze das Statusbit
	decfsz MENUE_COUNT		;
	goto $+3
	movlw d'5'
	movwf MENUE_COUNT
	movlw d'1'				;lade Zhler mit Wert 1
	movwf TEMP_DIG_CNT		;
	bsf TEMP_DIG_STAT,0		;setze Statusbit fr update Cursorposition in Menpunkten Goto, Position und Speed
	movlw d'250'			;Wartezeit von 250ms
	movwf LOOP_1			;
	call wait 
	goto main



;*******************************************************************************************
;	UNTERPROGRAMME
;*******************************************************************************************


inc_uz						;Unterprogramm zum erhhen der internen Normalzeituhr
	incf	SEK_UZ,f		; 1 Sekunden erhhen
	movlw	D'60'
	subwf	SEK_UZ,w
	btfss	STATUS,Z	
	goto	inc_uz_end		; 1 Sekunden <> 60	

	clrf	SEK_UZ
	incf	MIN_UZ,f		; 1 Minuten erhhen
	movlw	D'60'
	subwf	MIN_UZ,w
	btfss	STATUS,Z	
	goto	inc_uz_end		;  1 Minuten <> 60

	clrf	MIN_UZ
	incf	STD_UZ,f		; 1 Stunden erhhen
	movlw	D'24'
	subwf	STD_UZ,w
	btfss	STATUS,Z	
	goto	test_dcf_neu	; 1 Stunden <> 24

	clrf	STD_UZ
	incf	WTAG_UZ,f		; Wochentagszahl erhhen
	movlw	d'8'			; prfen ob Wert 8 erreicht ist
	subwf	WTAG_UZ,w
	btfss	STATUS,Z	
	goto	$+3				; Wert 8 noch nicht erreicht, weiter bei Tag erhhen
	movlw	d'1'			; Wert 8 ist erreicht, Rcksetzen der Wochentagszahl auf 1
	movwf	WTAG_UZ
	incf	TAG_UZ,f		; 1 Tag erhhen
	bsf		DISP_NEU,2

	movfw JAHR_UZ			;kopieren der aktiven Werte fr Jahr + Monat in die Temp Register
	movwf DCF_JAHR
	movfw MONAT_UZ
	movwf DCF_MONAT
	bsf PCLATH,4			;Select Page 2
	call monats_tage		;Aufruf des Unterprogramms zur Ermittlung der Tage eines Monats
	bcf PCLATH,4			;Select Page 0

test_tag					;die max Zahl der Tage des aktuellen Monats steht in W
	addlw 	0x01			;erhhe die Tage um 1
	subwf	TAG_UZ,w		;subtrahiere den aktuellen Tag und prfe ob Ergebnis =0
	btfss	STATUS,Z	
	goto	test_dcf_neu	; 1 Tag <> 28/29/30/31

	movlw	0x01			; Rcksetzen des Tags auf 1
	movwf	TAG_UZ
	incf	MONAT_UZ,f		; 1 Monat erhhen
	movlw	D'13'
	subwf	MONAT_UZ,w
	btfss	STATUS,Z	
	goto	inc_uz_end		; 1 Monat <> 12

	movlw	0x01			; Rcksetzen des Monats auf 1
	movwf	MONAT_UZ
	incf	JAHR_UZ,f		; 1 Jahr erhhen

test_dcf_neu
	movlw 0x14				;EEPROM Adresse fr Updatezeit
	call readEE				;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf PM_TEM			;Kopieren der Daten des W-Registers ins F-Register
	btfsc PM_TEM,7			;prfen ob Bit 7 fr Update auto/man nicht gesetzt ist
	goto inc_uz_end			;Bit 7 ist gesetzt (manuelles update), weiter zum Ende des Unterprogramms
	movfw STD_UZ			;Bit 7 ist nicht gesetzt (automatisches update) => hole aktuelle Stunde ins W-Register
	subwf PM_TEM,w			;subtrahiere die Updatezeit
	btfsc STATUS,Z			;prfen ob Ergebnis >0<
	bsf DCF_MESS_STAT,0		;Ergebnis = 0, setze das Update Bit

inc_uz_end
	bsf DISP_NEU,0			;setze Bit zur Aktualisierung der Uhrzeit im Display
	return

inc_sz						;Unterprogramm zum erhhen der internen Sternzeituhr
	incf	SEK_SZ,f		; 1 Sekunden erhhen
	movlw	D'60'
	subwf	SEK_SZ,w
	btfss	STATUS,Z	
	goto	inc_sz_end		; 1 Sekunden <> 10	

	clrf	SEK_SZ
	incf	MIN_SZ,f		; 1 Minuten erhhen
	movlw	D'60'
	subwf	MIN_SZ,w
	btfss	STATUS,Z	
	goto	inc_sz_end		;  1 Minuten <> 10

	clrf	MIN_SZ
	incf	STD_SZ,f		; 1 Stunden erhhen
	movlw	D'24'
	subwf	STD_SZ,w
	btfss	STATUS,Z	
	goto	inc_sz_end		; 1 Stunden <> 10

	clrf	STD_SZ

inc_sz_end
	bsf DISP_NEU,1			;setze Bit zur Aktualisierung der Sternzeit im Display
	return


tast_befehl						;Unterprogramm zum Laden und invertieren der Tastaturbefehle
	movlw 0xFF					;lade Register Befehl mit Wert 255
	movwf BEFEHL
	movfw PORTB					;hole PORTB ins W-Register und subtrahiere es von 255 im Register Befehl
	subwf BEFEHL				;255-PORTB=BEFEHL
	return


read_position					;Unterprogramm zum lesen der Positionsdaten (geogr. Lnge) aus dem internen EEPROM
	movlw 0x10					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf PM_TEM				;Kopieren der Daten des W-Registers ins F-Register, Ost=0, West=1
	movlw 0x11					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf GRD_TEM				;Kopieren der Daten des W-Registers ins F-Register, Winkelgrad
	movlw 0x12					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf MIN_TEM				;Kopieren der Daten des W-Registers ins F-Register, Winkelminuten
	movlw 0x13					;EEPROM Adresse
	call readEE					;Aufruf des Unterprogramms zum Lesen/Kopieren der EEPROM Werte ins W-Register
	movwf MIN_TEM_01			;Kopieren der Daten des W-Registers ins F-Register, Winkelsekunden
	return

readEE							;Unterprogramm zum lesen der Zelle deren Adresse in W steht (internes EEPROM)
	bsf STATUS, RP1 			;EEADR liegt in der Bank 2 
	bcf STATUS, RP0
 	movwf EEADR 				;schreibe die Adresse in EEADR 
	bsf STATUS, RP0				;Bank 3
	bcf EECON1, EEPGD			;Access data memory
	bsf EECON1, RD 				;EEPROM Leseproze starten 
	bcf STATUS, RP0 			;Bank 2 
	movf EEDATA, W 				;Die Daten der EEPROM Zelle nach W kopieren 
	bcf STATUS, RP1 			;Bank 0 
	return

writeEE							;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht (internes EEPROM)
	bsf STATUS, RP0				;Bank 3 (EECON1 ist in Bank 3)
	bsf STATUS, RP1
	btfsc EECON1, WR			;prfen, ob das EEPROM beschrieben werden kann
	goto $-1					;warten, bis das bit WR von der Hardware zurckgesetzt wurde
	bcf STATUS, RP0				;Bank 2 (EEADR und EEDATA sind in Bank 2)
	movwf EEADR					;Die Adresse an die die Daten geschrieben werden sollen ist im W-Register
	bcf STATUS, RP1				;Bank 0
	movfw TEMP_13				;hole zu speichernden Wert ins W-Register
	bsf STATUS, RP1				;Bank 2 (EEADR und EEDATA sind in Bank 2)
	movwf EEDATA				;Wert wird in EEADR geschrieben.
	bsf STATUS, RP0				;Bank 3
	bcf EECON1, EEPGD			;Das Bit EEPGD lschen, um das interne Daten-EEPROM auszuwhlen. Wird das Bit gesetzt, wird der Progamm-Flash ausgewhlt
	bsf EECON1, WREN			;erlaubt das Beschreiben des Daten-EEPROM
	bcf INTCON, GIE				;falls Interrupts verwendet werde, mssen diese abgeschaltet werden	
	movlw 0x55					;Um das EEPROM zu beschreiben, mssen die folgenden 5 Befehle
	movwf EECON2				;genau in dieser Reihenfolge aufgerufen werden.
	movlw 0xAA					;
	movwf EECON2				;
	bsf EECON1, WR				;Jetzt beginnt der eigentliche Schreibvorgang.
	bsf INTCON, GIE				;Die Interrupts knnen jetzt wieder freigegeben werden.
	bcf EECON1, WREN			;Das Beschreiben des EEPROM verbieten.
	bcf STATUS, RP0				;Bank 0
	bcf STATUS, RP1
	return						;Zurck ins Hauptprogramm


convert_2dig					;Unterprogramm zur Umwandlung von zB 1,2 in Zahl 12
	bcf STATUS, C				;Carrybit lschen, damit es beim rotieren nicht strt
	movfw TEMP_13				;hole die 10er Zahl ins W-Register und speichere sie im Rechenregister
	movwf TEMP_15				;mit 10 multiplizieren = x2x2x2+1x+1x=x10
	rlf TEMP_15,f				; x2
	rlf TEMP_15,f				; x2
	rlf TEMP_15,f				; x2
	movfw TEMP_13				;hole die 10er Zahl ins W-Register und addiere sie 2x zu dem Wert im Rechenregister
	addwf TEMP_15,f				;+1x
	addwf TEMP_15,f				;+1x, im Rechenregister steht nun der 10-fache Wert
	movfw TEMP_14				;hole die 1er Zahl ins W-Register und addiere sie zu dem Wert im Rechenregister
	addwf TEMP_15,f				;im Rechenregister steht nun der kombinierte Wert aus 10er und 1er 
	return


wait							;Unterprogramm fr Wartezeit (max. 250 ms)
	movlw d'250'
	movwf TEMP_13
loop_16							;loop_16 dauert 4us und wird 250 x ausgefhrt => 1000us = 1 ms
	nop
	nop
	nop
	nop
	nop
	decfsz TEMP_13
	goto loop_16
	decfsz LOOP_1				;LOOP_1 wird 1x pro ms reduziert
	goto wait
	return

;*******************************************************************************************
;	UNTERPROGRAMME fr die Mensteuerung
;*******************************************************************************************

menue_stat						;Unterprogramms fr Menue Flip Flop
	btfss MENUE_FF,0			;prfen ob bit 0 gesetzt ist
	goto menue_bitset			;wenn ja berspringe diesen Befehl, wenn nein setze bit in Register
	clrf MENUE_FF				;lsche Register
	movlw d'1'
	movwf MENUE_COUNT			;lade Register mit Wert 1
	movwf TEMP_DIG_CNT			;
	clrf MENUE_1_CNT			;lsche Register
	bsf PCLATH,3				;Select Page 1
	movlw 0x0C					;Befehl: Display On/Off Control (00001100)
	call disp_befehl	;
	call clear_display			;Aufruf des Unterprogramms zum Lschen des Displays
	movlw 0x06					;Befehl: Entry Mode Set (00000110)
	call disp_befehl	;
	bcf PCLATH,3				;Select Page 0
	movlw 0xFF					;setze bit 0-7 auf high (=255) in Display Statusregister und TEMP_DIG_STAT Register
	movwf DISP_NEU				;
	clrf TEMP_DIG_STAT			;
	bsf TEMP_DIG_STAT,0			;
	goto menue_stat_end
menue_bitset
	bsf MENUE_FF,0				;setze bit in Register
	bsf DISP_NEU,4
menue_stat_end
	movlw d'250'				;Wartezeit von 250ms
	movwf LOOP_1				;
	call wait 		
	return

inc_dig_temp_cnt				;Unterprogramm zur Erhhung des Menpunktes horizontal
	movfw TEMP_CNT_MAX			;
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z				;prfen ob Zhler gleich dem Maximalwert ist
	goto $+4					;wenn ja, berspringe diesen Befehl	
	movlw d'1'
	movwf TEMP_DIG_CNT
	goto inc_temp_cnt_end
	incf TEMP_DIG_CNT
inc_temp_cnt_end
	bsf TEMP_DIG_STAT,0			;setze das Status bit
	return		

dec_dig_temp_cnt				;Unterprogramm zur Reduzierung des Menpunktes horizontal
	decfsz TEMP_DIG_CNT			;reduziere Zhler um 1 und prfe ob der der Zhler 0 ist
	goto dec_temp_cnt_end		;wenn ja, berspringe diesen Befehl	
	movfw TEMP_CNT_MAX			;
	movwf TEMP_DIG_CNT
dec_temp_cnt_end
	bsf TEMP_DIG_STAT,0			;setze das Status bit
	return

dig_temp_kat					;Unterprogramm zum setzen des Cursor Position in den Hauptmenueunterpunkten
	movlw d'1'					;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w		;
	btfss STATUS,Z				;prfen ob Ergebniss =0		
	goto dig_temp_kat2			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movlw 0xC1					;lade Adresse ins W-Register (2.Zeichen in der 2.Reihe)
	movwf TEMP_DIG_ADR			;speichere Adresse im temp-Register
	goto dig_temp_katend		;springe zum Ende des Unterprogramms
dig_temp_kat2
	movlw d'2'
	subwf TEMP_DIG_CNT,w		;
	btfss STATUS,Z				;prfen ob Ergebniss =0		
	goto dig_temp_kat3			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movlw 0xC5					;lade Adresse ins W-Register (6.Zeichen in der 2.Reihe)
	movwf TEMP_DIG_ADR			;speichere Adresse im temp-Register
	goto dig_temp_katend		;springe zum Ende des Unterprogramms
dig_temp_kat3
	movlw d'3'
	subwf TEMP_DIG_CNT,w		;
	btfss STATUS,Z				;prfen ob Ergebniss =0		
	goto dig_temp_katend		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	movlw 0xCA					;lade Adresse ins W-Register (11.Zeichen in der 2.Reihe)
	movwf TEMP_DIG_ADR			;speichere Adresse im temp-Register
	goto dig_temp_katend		;springe zum Ende des Unterprogramms
dig_temp_katend
	bsf PCLATH,3				;Select Page 1
	movfw TEMP_DIG_ADR			;Adresse fr das zu schreibende Zeichen
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw 0x20					;Daten fr das zu schreibende Zeichen (blank)
	call disp_daten				;bertrage das Zeichen
	bcf PCLATH,3				;Select Page 0
	bcf TEMP_DIG_STAT,0			;lsche das Status bit
	movlw d'250'				;Wartezeit von 250ms
	movwf LOOP_1				;
	call wait 		
	return		


;para_routine: Unterprogramm zur Berechnung der Mittelwerte fr L/H Bit des DCF-Signals
;die folgenden temporren Register werden verwendet:
;MIN_TEM_01		= DCF LBit: Summe der DCF LBits, L-Register
;MIN_TEM		= DCF LBit: Summe der DCF LBits, H-Register
;GRD_TEM		= DCF LBit: Anzahl der DCF LBits
;PM_TEM			= DCF LBit: Mittelwert der DCF LBits
;STD_TEMP		= DCF HBit: Summe der DCF HBits, L-Register
;MIN_TEMP		= DCF HBit: Summe der DCF HBits, H-Register
;SEK_TEMP		= DCF HBit: Anzahl der DCF HBits
;REST_SEK_TEMP	= DCF HBit: Mittelwert der DCF HBits
;DCF_PRUEFBIT	= Unterscheidung zwischen L/H

para_routine
	bcf DCF_MESS_STAT,4		;lsche Statusbit 
	btfss DCF_MESS_STAT,3	;wenn statusbit fr neuen datensatz = high, berspringe nchsten Befehl 
	goto para_routine_end
	movfw TBIT_H_TEMP		;prfen ob Bitlnge >255 (berlauf von TBIT_L_TEMP auf TBIT_H_TEMP)
	btfss STATUS,Z
	goto para_routine_end	;Bitlnge ist >255, gehe zum Ende des Unterprogramms
	movlw d'30'				;prfen ob Bitlnge <30
	subwf TBIT_L_TEMP,w
	btfss STATUS,C			;
	goto para_routine_end	;Bitlnge ist <30, gehe zum Ende des Unterprogramms
	movfw DCF_PRUEFBIT		;
	subwf TBIT_L_TEMP,w
	btfsc STATUS,C
	goto analyse_h
	movlw d'20'				;wenn 20 Low Bits ausgewertet sind, setze Statusbit DCF_MESS_STAT,5 und gehe zum Unterprogrammende
	subwf GRD_TEM,w
	btfss STATUS,Z
	goto $+3				;Anzahl Low Bits <20, weiter in der Auswertung
	bsf DCF_MESS_STAT,5		;Anzahl Low Bits =20, springe zum Ende des Unterprogramms
	goto para_routine_end	
	incf GRD_TEM			;erhhe Zhler fr Anzahl Low Bits
	movfw TBIT_L_TEMP		;addiere Bitlnge zum L-Register
	addwf MIN_TEM_01
	btfsc STATUS,C			;prfen, ob L-Register bergelaufen ist
	incf MIN_TEM			;Es ist ein berlauf aufgetreten, erhhe H-Register  
	movfw MIN_TEM_01		;kopiere Zhlerwerte fr die Division
	movwf TEMP_11
	movfw MIN_TEM
	movwf TEMP_12
	clrf TEMP_13
	movfw GRD_TEM			;kopiere Nennerwerte fr die Division
	movwf TEMP_15
	clrf TEMP_16
	bsf PCLATH,4			;Select Page 2
	call Div16				;Aufruf des Unterprogramms zur Division Summe Bitlnge/Anzahl Bits
	bcf PCLATH,4			;Select Page 0			
	movfw TEMP_11			;kopiere den berechneten Mittelwert in das Mittelwert Register
	movwf PM_TEM
	movwf DCF_PRUEFBIT
	bcf STATUS,C
	rrf TEMP_11,w
	addwf DCF_PRUEFBIT
	goto para_routine_end
analyse_h
	movlw d'20'				;wenn 20 High Bits ausgewertet sind, setze Statusbit DCF_MESS_STAT,6 und gehe zum Unterprogrammende
	subwf SEK_TEMP,w
	btfss STATUS,Z
	goto $+3				;Anzahl High Bits <20, weiter in der Auswertung
	bsf DCF_MESS_STAT,6		;Anzahl High Bits =20, springe zum Ende des Unterprogramms
	goto para_routine_end		
	incf SEK_TEMP			;erhhe Zhler fr Anzahl High Bits
	movfw TBIT_L_TEMP		;addiere Bitlnge zum H-Register
	addwf STD_TEMP
	btfsc STATUS,C			;prfen, ob L-Register bergelaufen ist
	incf MIN_TEMP			;Es ist ein berlauf aufgetreten, erhhe H-Register  
	movfw STD_TEMP			;kopiere Zhlerwerte fr die Division
	movwf TEMP_11
	movfw MIN_TEMP
	movwf TEMP_12
	clrf TEMP_13
	movfw SEK_TEMP			;kopiere Nennerwerte fr die Division
	movwf TEMP_15
	clrf TEMP_16
	bsf PCLATH,4			;Select Page 2
	call Div16				;Aufruf des Unterprogramms zur Division Summe Bitlnge/Anzahl Bits
	bcf PCLATH,4			;Select Page 0			
	movfw TEMP_11			;kopiere den berechneten Mittelwert in das Mittelwert Register
	movwf REST_SEK_TEMP
para_routine_end
	bsf PCLATH,3			;Select Page 1	
	call disp_set_para2
	bcf PCLATH,3			;Select Page 0
	return

dcf_minmax					;Unterprogramm zur Berechnung der Toleranzen des DCF-Signals:
							;Low Bit: Mittelwert Low Bit +/- 25%
							;High Bit: Mittelwert High Bit +/- 12,5%
	movfw PM_TEM			;kopiere Mittelwert Low Bit ins TEMP_11 Register und verschiebe 
	movwf TEMP_11			;es 2x nach rechts (=/4 => 25%)
	bcf STATUS,C
	rrf TEMP_11,f
	bcf STATUS,C
	rrf TEMP_11,f
	movfw TEMP_11			;hole den 25% Wert ins W-Register und subtrahiere in vom Mittelwert
	subwf PM_TEM,w
	movwf MIN_TEM_01		;der Low Bit min Wert steht nun im temporren Register MIN_TEM_01
	movfw TEMP_11			;hole den 25% Wert ins W-Register und addiere den Mittelwert
	addwf PM_TEM,f			;der Low Bit max Wert steht nun im temporren Register PM_TEM

	movfw REST_SEK_TEMP		;kopiere Mittelwert High Bit ins TEMP_11 Register und verschiebe 
	movwf TEMP_11			;es 3x nach rechts (=/8 => 12,5%)
	bcf STATUS,C
	rrf TEMP_11,f
	bcf STATUS,C
	rrf TEMP_11,f
	bcf STATUS,C
	rrf TEMP_11,f
	movfw TEMP_11			;hole den 12,5% Wert ins W-Register und subtrahiere in vom Mittelwert
	subwf REST_SEK_TEMP,w
	movwf STD_TEMP			;der High Bit min Wert steht nun im temporren Register STD_TEMP
	movfw TEMP_11			;hole den 25% Wert ins W-Register und addiere den Mittelwert
	addwf REST_SEK_TEMP,f	;der High Bit max Wert steht nun im temporren Register REST_SEK_TEMP
	btfss STATUS,C			;prfen, ob REST_SEK_TEMP-Register bergelaufen ist
	goto $+3				;Register ist nicht bergelaufen
	movlw 0xFF				;Register ist bergelaufen, berschreibe den Wert mit 255
	movwf REST_SEK_TEMP
	bsf PCLATH,3			;Select Page 1	
	call disp_set_para3
	bcf PCLATH,3			;Select Page 0
	return	




;*******************************************************************************************
;	PAGE 1, 	bsf PCLATH,3				;Select Page 1
;				bcf PCLATH,4
;*******************************************************************************************
	ORG 0x800					;PAGE 1

;*******************************************************************************************
; Weitere Unterprogramme zur Mensteuerung
;*******************************************************************************************

dig_datum1					;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt Datum/Uhrzeit manuell
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_datum2			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfsc TEMP_DIG_STAT,0	;prfen ob die Cursor Position sich gendert hat
	goto cursor_datum1		;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_datum1			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf DCF_TAG			;Bit ist gesetzt, erhhe Zhler um 1
	movlw d'32'				;lade Wert 32 ins W-Register
	subwf DCF_TAG,w			;subtrahiere Wert nach Erhhung
	btfss STATUS,Z			;prfen ob Ergebnis =0
 	goto cursor_datum1		;
	movlw d'1'				;Rcksetzen des Registers mit Wert 1	
	movwf DCF_TAG			;
	goto cursor_datum1
dec_datum1
	decfsz DCF_TAG			;Reduziere Register und prfe ob 0
	goto cursor_datum1		;Wert >0
	movlw d'31'				;Rcksetzen des Registers 
	movwf DCF_TAG
cursor_datum1
	movlw 0x84				;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR		;speichere Adresse im temp-Register
	movfw DCF_TAG			;hole Wert in temporres Register
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_15
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_DIG_VAL
	movfw TEMP_14			;schiebe Wert in temporres Display Register fr Wert der vorhergehenden Adresse
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_VAL_PREV
	goto dig_datum_end		;springe zum Ende des Unterprogramms
dig_datum2
	movlw d'2'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_datum3
	btfsc TEMP_DIG_STAT,0	;prfen ob die Cursor Position sich gendert hat
	goto cursor_datum2		;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_datum2			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf DCF_MONAT
	movlw d'13'
	subwf DCF_MONAT,w
	btfss STATUS,Z			;prfen ob Ergebnis =0
 	goto cursor_datum2 		;springe zum Ende des Unterprogramms
	movlw d'1'				;Rcksetzen des Registers mit Wert 1	
	movwf DCF_MONAT			;
	goto cursor_datum2
dec_datum2
	decfsz DCF_MONAT		;Reduziere Register und prfe ob 0
	goto cursor_datum2		;Wert >0,
	movlw d'12'				;Rcksetzen des Registers 
	movwf DCF_MONAT
cursor_datum2
	movlw 0x87				;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR		;speichere Adresse im temp-Register
	movfw DCF_MONAT			;
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_15
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_DIG_VAL
	movfw TEMP_14			;schiebe Wert in temporres Display Register fr Wert der vorhergehenden Adresse
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_VAL_PREV
	goto dig_datum_end
dig_datum3
	movlw d'3'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_datum4
	btfsc TEMP_DIG_STAT,0	;prfen ob die Cursor Position sich gendert hat
	goto cursor_datum3		;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_datum3			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf DCF_JAHR
	movlw d'100'
	subwf DCF_JAHR,w
	btfsc STATUS,Z		
	clrf DCF_JAHR
	goto cursor_datum3
dec_datum3
	movfw DCF_JAHR			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf DCF_JAHR			;reduziere Zhler um 1
	goto $+3				;schiebe Wert in temporres Display Register
	movlw d'99'
	movwf DCF_JAHR
cursor_datum3
	movlw 0x8A				;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR		;speichere Adresse im temp-Register
	movfw DCF_JAHR
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_15
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_DIG_VAL
	movfw TEMP_14			;schiebe Wert in temporres Display Register fr Wert der vorhergehenden Adresse
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_VAL_PREV
	goto dig_datum_end
dig_datum4
	movlw d'4'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_datum5
	btfsc TEMP_DIG_STAT,0	;prfen ob die Cursor Position sich gendert hat
	goto cursor_datum4		;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_datum4			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf DCF_WTAG
	movlw d'8'
	subwf DCF_WTAG,w
	btfss STATUS,Z			;prfen ob Ergebnis =0
 	goto cursor_datum4 		;
	movlw d'1'				;Rcksetzen des Registers mit Wert 1	
	movwf DCF_WTAG			;
	goto cursor_datum4
dec_datum4
	decfsz DCF_WTAG			;Reduziere Register und prfe ob 0
	goto cursor_datum4		;Wert >0, 
	movlw d'7'				;Rcksetzen des Registers 
	movwf DCF_WTAG
cursor_datum4
	movlw 0x8D				;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR		;speichere Adresse im temp-Register
	movfw DCF_WTAG
	movwf TEMP_13
	call wochentag			;Aufruf des Unterprogramms wochentag
	movfw TEMP_15
	movwf TEMP_DIG_VAL
	movfw TEMP_14			;schiebe Wert in temporres Display Register fr Wert der vorhergehenden Adresse
	movwf TEMP_VAL_PREV
	goto dig_datum_end
dig_datum5
	movlw d'5'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_datum6
	btfsc TEMP_DIG_STAT,0	;prfen ob die Cursor Position sich gendert hat
	goto cursor_datum5		;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_datum5			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf DCF_STD
	movlw d'24'
	subwf DCF_STD,w
	btfsc STATUS,Z		
	clrf DCF_STD
	goto cursor_datum5
dec_datum5
	movfw DCF_STD			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf DCF_STD			;reduziere Zhler um 1
	goto $+3				;schiebe Wert in temporres Display Register
	movlw d'23'
	movwf DCF_STD
cursor_datum5
	movlw 0xC4				;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR		;speichere Adresse im temp-Register
	movfw DCF_STD
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_15
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_DIG_VAL
	movfw TEMP_14			;schiebe Wert in temporres Display Register fr Wert der vorhergehenden Adresse
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_VAL_PREV
	goto dig_datum_end
dig_datum6
	movlw d'6'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_datum7
	btfsc TEMP_DIG_STAT,0	;prfen ob die Cursor Position sich gendert hat
	goto cursor_datum6		;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_datum6			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf DCF_MIN
	movlw d'60'
	subwf DCF_MIN,w
	btfsc STATUS,Z		
	clrf DCF_MIN
	goto cursor_datum6
dec_datum6
	movfw DCF_MIN			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf DCF_MIN			;reduziere Zhler um 1
	goto $+3				;schiebe Wert in temporres Display Register
	movlw d'59'
	movwf DCF_MIN
cursor_datum6
	movlw 0xC7				;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR		;speichere Adresse im temp-Register
	movfw DCF_MIN
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_15
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_DIG_VAL
	movfw TEMP_14			;schiebe Wert in temporres Display Register fr Wert der vorhergehenden Adresse
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_VAL_PREV
	goto dig_datum_end
dig_datum7
	movlw d'7'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_datum8
	btfsc TEMP_DIG_STAT,0	;prfen ob die Cursor Position sich gendert hat
	goto cursor_datum7		;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_datum7			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf SEK_TEMP
	movlw d'60'
	subwf SEK_TEMP,w
	btfsc STATUS,Z		
	clrf SEK_TEMP
	goto cursor_datum7
dec_datum7
	movfw SEK_TEMP			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf SEK_TEMP			;reduziere Zhler um 1
	goto $+3				;schiebe Wert in temporres Display Register
	movlw d'59'
	movwf SEK_TEMP
cursor_datum7
	movlw 0xCA				;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR		;speichere Adresse im temp-Register
	movfw SEK_TEMP
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_15
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_DIG_VAL
	movfw TEMP_14			;schiebe Wert in temporres Display Register fr Wert der vorhergehenden Adresse
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_VAL_PREV
	goto dig_datum_end
dig_datum8
	movlw d'8'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_datum9
	btfsc TEMP_DIG_STAT,0	;prfen ob die Cursor Position sich gendert hat
	goto cursor_datum8		;Cursor Position hat sich verndert, gehe zu update Cursor
	movlw A'+'
	subwf PM_TEM,w
	btfss STATUS,Z	
	goto $+3	
	movlw A'-'
	goto $+2
	movlw A'+'
	movwf PM_TEM
cursor_datum8
	movlw 0xCD				;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR		;speichere Adresse im temp-Register
	movfw PM_TEM
	movwf TEMP_DIG_VAL
	movlw A'Z'
	movwf TEMP_VAL_PREV		;schiebe Wert in temporres Display Register fr Wert der vorhergehenden Adresse
	goto dig_datum_end
dig_datum9
	movlw d'9'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_datum_end
	btfsc TEMP_DIG_STAT,0	;prfen ob die Cursor Position sich gendert hat
	goto cursor_datum9		;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_datum9			;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf DCF_MESZ
	movlw d'13'
	subwf DCF_MESZ,w
	btfsc STATUS,Z		
	clrf DCF_MESZ
	goto cursor_datum9
dec_datum9
	movfw DCF_MESZ			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf DCF_MESZ			;reduziere Zhler um 1
	goto $+3				;schiebe Wert in temporres Display Register
	movlw d'12'
	movwf DCF_MESZ
cursor_datum9
	movlw 0xCF				;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR		;speichere Adresse im temp-Register
	movfw DCF_MESZ			;
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_15
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_DIG_VAL
	movfw TEMP_14			;schiebe Wert in temporres Display Register fr Wert der vorhergehenden Adresse
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_VAL_PREV
	goto dig_datum_end
dig_datum_end
	movfw TEMP_DIG_ADR			;Adresse fr das zu schreibende Zeichen
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_DIG_VAL			;Daten fr das zu schreibende Zeichen, untere 4 bit
	call disp_daten		;bertrage das Zeichen
	movfw TEMP_DIG_ADR			;Adresse fr das zu schreibende Zeichen
	decf TEMP_DIG_ADR,w			;Reduziere die Adresse um 1
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_VAL_PREV			;Daten fr das zu schreibende Zeichen
	call disp_daten		;bertrage das Zeichen
	bcf TEMP_DIG_STAT,0			;lsche das Status bit
	movlw d'250'				;Wartezeit von 250ms
	movwf LOOP_1				;
	bcf PCLATH,3				;Select Page 0
	call wait 		
	bsf PCLATH,3				;Select Page 1
	clrf TEMP_DIG_STAT			;lsche Statusbit Register
	return

dig_standort1				;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt Standort
	movlw d'1'				;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w	;
	btfss STATUS, Z			;prfen ob Ergebniss =0		
	goto dig_standort2		;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfsc TEMP_DIG_STAT,0	;prfen ob die Cursor Position sich gendert hat
	goto cursor_standort1	;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_standort1		;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf GRD_TEM
	movlw d'180'
	subwf GRD_TEM,w
	btfsc STATUS,Z		
	clrf GRD_TEM
	goto cursor_standort1
dec_standort1
	movfw GRD_TEM			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf GRD_TEM			;reduziere Zhler um 1
	goto $+3				;schiebe Wert in temporres Display Register
	movlw d'179'
	movwf GRD_TEM
cursor_standort1
	movlw 0xC4				;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR		;speichere Adresse im temp-Register
	movfw GRD_TEM			;schiebe Wert in temporres Display Register
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movlw 0xC2				;Adresse fr das zu schreibende Zeichen, 100er
	call disp_befehl	;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_13			;Daten fr das zu schreibende Zeichen, 100er, untere 4 bit
	addlw b'00110000'		;addiere obere 4 bit
	call disp_daten	;bertrage das Zeichen	
	movfw TEMP_15
	movwf TEMP_DIG_VAL
	movfw TEMP_14			;schiebe Wert in temporres Display Register fr Wert der vorhergehenden Adresse
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_VAL_PREV
	goto dig_standort_end	;springe zum Ende des Unterprogramms
dig_standort2
	movlw d'2'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_standort3
	btfsc TEMP_DIG_STAT,0	;prfen ob die Cursor Position sich gendert hat
	goto cursor_standort2	;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_standort2		;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf MIN_TEM
	movlw d'60'
	subwf MIN_TEM,w
	btfsc STATUS,Z		
	clrf MIN_TEM
	goto cursor_standort2
dec_standort2
	movfw MIN_TEM			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf MIN_TEM			;reduziere Zhler um 1
	goto $+3				;schiebe Wert in temporres Display Register
	movlw d'59'
	movwf MIN_TEM
cursor_standort2
	movlw 0xC7				;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR		;speichere Adresse im temp-Register
	movfw MIN_TEM			;schiebe Wert in temporres Display Register
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_15
	movwf TEMP_DIG_VAL
	movfw TEMP_14			;schiebe Wert in temporres Display Register fr Wert der vorhergehenden Adresse
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_VAL_PREV
	goto dig_standort_end
dig_standort3
	movlw d'3'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_standort4
	btfsc TEMP_DIG_STAT,0	;prfen ob die Cursor Position sich gendert hat
	goto cursor_standort3	;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_standort3		;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf MIN_TEM_01
	movlw d'60'
	subwf MIN_TEM_01,w
	btfsc STATUS,Z		
	clrf MIN_TEM_01
	goto cursor_standort3
dec_standort3
	movfw MIN_TEM_01		;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf MIN_TEM_01			;reduziere Zhler um 1
	goto $+3				;schiebe Wert in temporres Display Register
	movlw d'59'
	movwf MIN_TEM_01
cursor_standort3
	movlw 0xCA				;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR		;speichere Adresse im temp-Register
	movfw MIN_TEM_01		;Wert fr Bogensekunden
	call convert_dig		;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_15
	movwf TEMP_DIG_VAL
	movfw TEMP_14			;schiebe Wert in temporres Display Register fr Wert der vorhergehenden Adresse
	addlw b'00110000'		;addiere obere 4 bit
	movwf TEMP_VAL_PREV
	goto dig_standort_end
dig_standort4
	movlw d'4'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_standort_end
	btfsc TEMP_DIG_STAT,0	;prfen ob die Cursor Position sich gendert hat
	goto cursor_standort4	;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss PM_TEM,0			;0 = Ost, 1 = West
	goto $+3
	bcf PM_TEM,0
	goto $+2
	bsf PM_TEM,0
cursor_standort4
	movlw 0xCD				;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR		;speichere Adresse im temp-Register
	movlw  b'00110000'		;obere 4 bit
	btfss PM_TEM,0			;prfen ob PM_TEM = 1 (West)
	sublw A'E'				;PM_TEM = 0 (Ost), subtrahiere b'00110000' von A'E'
	btfsc PM_TEM,0			;prfen ob PM_TEM = 0 (Ost)
	sublw A'W'				;PM_TEM = 1 (West), subtrahiere b'00110000' von A'W'
	movwf TEMP_DIG_VAL
	movlw 0x20				;schiebe Wert "blank" in temporres Display Register fr Wert der vorhergehenden Adresse
	movwf TEMP_VAL_PREV
	goto dig_standort_end
dig_standort_end
	movfw TEMP_DIG_ADR			;Adresse fr das zu schreibende Zeichen
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_DIG_VAL			;Daten fr das zu schreibende Zeichen, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_DIG_ADR			;Adresse fr das zu schreibende Zeichen
	decf TEMP_DIG_ADR,w			;Reduziere die Adresse um 1
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_VAL_PREV			;Daten fr das zu schreibende Zeichen
	call disp_daten				;bertrage das Zeichen
	bcf TEMP_DIG_STAT,0			;lsche das Status bit
	movlw d'250'				;Wartezeit von 250ms
	movwf LOOP_1				;
	bcf PCLATH,3				;Select Page 0
	call wait 		
	bsf PCLATH,3				;Select Page 1
	clrf TEMP_DIG_STAT			;lsche Statusbit Register
	return

dig_dcfup_aut
	btfss TEMP_DIG_STAT,1	;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto dec_dcfup_aut		;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf PM_TEM
	movlw d'24'
	subwf PM_TEM,w
	btfsc STATUS,Z		
	clrf PM_TEM
	goto cursor_dcfup_aut
dec_dcfup_aut
	movfw PM_TEM			;hole Wert ins W-Register
	btfsc STATUS,Z			;prfe ob Wert <0>
	goto $+3				;wenn ja berspringe diesen Befehl, wenn nein setze Register auf 0
	decf PM_TEM				;reduziere Zhler um 1
	goto $+3				;schiebe Wert in temporres Display Register
	movlw d'23'
	movwf PM_TEM
cursor_dcfup_aut
	movlw 0x8C					;Adresse fr das zu schreibende Zeichen
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw PM_TEM
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_15				;Daten fr das zu schreibende Zeichen, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x8B					;Adresse fr das zu schreibende Zeichen
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_14				;Daten fr das zu schreibende Zeichen, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw d'250'				;Wartezeit von 250ms
	movwf LOOP_1				;
	bcf PCLATH,3				;Select Page 0
	call wait 		
	bsf PCLATH,3				;Select Page 1
	clrf TEMP_DIG_STAT			;lsche Statusbit Register
	return


dig_para_man1					;Unterprogramm zur Erhhung/Reduzierung der Werte im Menuepunkt Parameter manuell
	movlw d'1'					;prfen ob der 1. Wert verndert werden soll,
	subwf TEMP_DIG_CNT,w		;
	btfss STATUS, Z				;prfen ob Ergebniss =0		
	goto dig_para_man2			;wenn ja, berspringe diesen Befehl, wenn nein weiter bei nchster Prfung
	btfsc TEMP_DIG_STAT,0		;prfen ob die Cursor Position sich gendert hat
	goto cursor_para1			;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1		;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto $+3					;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf MIN_TEM_01				;erhhe L-Bit min
	goto cursor_para1
	decf MIN_TEM_01				;reduziere  L-Bit min
cursor_para1
	movlw 0x84					;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR			;speichere Adresse im temp-Register
	movfw MIN_TEM_01			;schiebe L-Bit min ins W-Register
	goto dig_para_end			;springe zum Ende des Unterprogramms
dig_para_man2
	movlw d'2'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_para_man3
	btfsc TEMP_DIG_STAT,0		;prfen ob die Cursor Position sich gendert hat
	goto cursor_para2			;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1		;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto $+3					;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf PM_TEM					;erhhe L-Bit max
	goto cursor_para2
	decf PM_TEM					;reduziere  L-Bit max
cursor_para2
	movlw 0x8C					;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR			;speichere Adresse im temp-Register
	movfw PM_TEM				;schiebe L-Bit max ins W-Register
	goto dig_para_end			;springe zum Ende des Unterprogramms
dig_para_man3
	movlw d'3'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_para_man4
	btfsc TEMP_DIG_STAT,0		;prfen ob die Cursor Position sich gendert hat
	goto cursor_para3			;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1		;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto $+3					;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf STD_TEMP				;erhhe H-Bit min
	goto cursor_para3
	decf STD_TEMP				;reduziere  H-Bit min
cursor_para3
	movlw 0xC4					;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR			;speichere Adresse im temp-Register
	movfw STD_TEMP				;schiebe H-Bit min ins W-Register
	goto dig_para_end			;springe zum Ende des Unterprogramms
dig_para_man4
	movlw d'4'
	subwf TEMP_DIG_CNT,w
	btfss STATUS, Z		
	goto dig_para_end
	btfsc TEMP_DIG_STAT,0		;prfen ob die Cursor Position sich gendert hat
	goto cursor_para4			;Cursor Position hat sich verndert, gehe zu update Cursor
	btfss TEMP_DIG_STAT,1		;prfen ob Bit fr Erhhung des zu ndernden Wertes gesetzt ist
	goto $+3					;Bit ist nicht gesetzt, gehe zu Reduzierung des zu ndernden Wertes
	incf REST_SEK_TEMP			;erhhe H-Bit max
	goto cursor_para4
	decf REST_SEK_TEMP			;reduziere  H-Bit max
cursor_para4
	movlw 0xCC					;lade Adresse ins W-Register
	movwf TEMP_DIG_ADR			;speichere Adresse im temp-Register
	movfw REST_SEK_TEMP			;schiebe H-Bit max ins W-Register
	goto dig_para_end			;springe zum Ende des Unterprogramms
dig_para_end
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_DIG_ADR			;Adresse fr das zu schreibende Zeichen
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_13				;Daten fr das zu schreibende Zeichen,100er untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten fr das zu schreibende Zeichen,10er untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr das zu schreibende Zeichen,1er untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	;die Adresse wird 2x erhht, damit der Cursor auf dem letzten Zeichen steht
	incf TEMP_DIG_ADR			;erhhe Adresse fr das zu schreibende Zeichen
	incf TEMP_DIG_ADR			;erhhe Adresse fr das zu schreibende Zeichen
	movfw TEMP_DIG_ADR			;Adresse fr das zu schreibende Zeichen
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	call cursor_on
	movlw d'250'				;Wartezeit von 250ms
	movwf LOOP_1				;
	bcf PCLATH,3				;Select Page 0
	call wait 		
	bsf PCLATH,3				;Select Page 1
	clrf TEMP_DIG_STAT			;lsche Statusbit Register
	return

;*******************************************************************************************
; Unterprogramme fr die Anzeige auf dem LCD Display
;*******************************************************************************************

disp_menue_1
	call clear_display		;Aufruf des Unterprogramms zum Lschen des Displays
	movlw d'1'				;
	subwf MENUE_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menue_2
	movlw d'2'				;Anzeige hat 2 Stellen im Untermen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	call dispm1_dcfupdate
	goto disp_menue_end
disp_menue_2
	movlw d'2'				;
	subwf MENUE_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menue_3
	movlw d'1'				;Anzeige hat 1 Stelle, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call cursor_off			;Befehl: Display On/Off Control, Cursor OFF (00001100)
	call dispm1_standort
	goto disp_menue_end
disp_menue_3
	movlw d'3'				;
	subwf MENUE_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menue_4
	movlw d'1'				;Anzeige hat 1 Stelle, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call cursor_off			;Befehl: Display On/Off Control, Cursor OFF (00001100)
	call dispm1_datumzeit
	goto disp_menue_end
disp_menue_4
	movlw d'4'				;
	subwf MENUE_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menue_5
	movlw d'2'				;Anzeige hat 2 Stellen im Untermen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call cursor_on			;Befehl: Display On/Off Control, Cursor ON (00001110)
	call dispm1_parameter
	goto disp_menue_end
disp_menue_5
	movlw d'5'				;
	subwf MENUE_COUNT, w	;
	btfss STATUS,Z		
	goto disp_menue_end
	movlw d'1'				;Anzeige hat 1 Stellen im Untermen, die gesetzt werden
	movwf TEMP_CNT_MAX		;
	call cursor_off			;Befehl: Display On/Off Control, Cursor OFF (00001100)
	call dispm1_q1hz
disp_menue_end
	bcf DISP_NEU,4
	return

dispm1_dcfupdate				;Anzeige Menpunkt: 1.DCF77 Update
	movlw 0x80					;Zeichen 1 in der 1. Zeile 
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'1'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'C'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'7'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'7'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'U'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'p'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0xC2					;Zeichen 3 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'M'					;ASCII:S
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:Z
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'A'					;ASCII:S
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:Z
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return	

dispm1_standort					;Anzeige Menpunkt: 2.Standort
	movlw 0x80					;Zeichen 1 in der 1. Zeile 
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'2'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'S'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return	

dispm1_datumzeit				;Anzeige Menpunkt: 3.Datum/Zeit man
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'3'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'/'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'Z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return	

dispm1_parameter				;Anzeige Menpunkt: 4.DCF Parameter
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'4'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'D'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'C'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'F'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'P'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0xC2					;Zeichen 3 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'M'					;ASCII:S
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:Z
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'A'					;ASCII:S
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:Z
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return	

dispm1_q1hz
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'5'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'Q'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'r'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'b'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'c'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'h'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_uhr
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'U'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'Z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movfw STD_UZ				;Daten fr Stunde
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr Stunde 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr Stunde 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movfw MIN_UZ				;Daten fr Minute
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr Minute 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr Minute 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movfw SEK_UZ				;Daten fr Sekunde
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr Sekunde 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr Sekunde 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	btfss ZZ_UZ,7				;prfen ob Bit 7 (=Vorzeichen +/-) im Register Zeitzone gesetzt ist: 1=-, 0=+ 
	goto $+3
	movlw A'-'
	goto $+2
	movlw A'+'
	call disp_daten				;bertrage das Zeichen
	movfw ZZ_UZ					;kopiere ZZ_UZ ins TEMP_11 Register und lsche Bit 7 (=Vorzeichen +/-)
	movwf TEMP_11
	bcf TEMP_11,7
	movfw TEMP_11
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	bcf DISP_NEU,0
	return

disp_sz
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'S'					;ASCII:S
	call disp_daten				;bertrage das Zeichen
	movlw A'Z'					;ASCII:Z
	call disp_daten				;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Daten fr "blank"
	call disp_daten				;bertrage das Zeichen
	movfw STD_SZ				;Daten fr Stunde
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr Stunde 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr Stunde 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movfw MIN_SZ				;Daten fr Minute
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr Minute 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr Minute 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movfw SEK_SZ				;Daten fr Sekunde
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr Sekunde 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr Sekunde 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	bcf DISP_NEU,1
	return

disp_datum
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw WTAG_UZ				;kopiere die Wochentagszahl ins TEMP_13 Register 
	movwf TEMP_13				;
	call wochentag				;Aufruf des Unterprogramms wochentag
	movfw TEMP_14				;Wochentag, 1.Ziffer
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Wochentag, 2.Ziffer
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	movfw TAG_UZ				;Daten fr Tag
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr Stunde 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr Stunde 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;Daten fr "."
	call disp_daten				;bertrage das Zeichen
	movfw MONAT_UZ				;Daten fr Monat
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr Stunde 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr Stunde 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;Daten fr "."
	call disp_daten				;bertrage das Zeichen
	movfw JAHR_UZ				;Daten fr Jahr
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr Stunde 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr Stunde 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	bcf DISP_NEU,2
	return

wochentag						;Unterprogramm zur Anzeige der 2 Ziffern des Wochentags (TEMP_14/TEMP_15)
	movlw 0x01					;aus der Wochentagszahl (TEMP_13), 1 = MO, 2 = DI, .... 7 = SO
	subwf TEMP_13,w
	btfss STATUS,Z
	goto $+6
	movlw A'M'
	movwf TEMP_14
	movlw A'O'
	movwf TEMP_15
	goto wochentag_end

	movlw 0x02
	subwf TEMP_13,w
	btfss STATUS,Z
	goto $+6
	movlw A'D'
	movwf TEMP_14
	movlw A'I'
	movwf TEMP_15
	goto wochentag_end

	movlw 0x03
	subwf TEMP_13,w
	btfss STATUS,Z
	goto $+6
	movlw A'M'
	movwf TEMP_14
	movlw A'I'
	movwf TEMP_15
	goto wochentag_end

	movlw 0x04
	subwf TEMP_13,w
	btfss STATUS,Z
	goto $+6
	movlw A'D'
	movwf TEMP_14
	movlw A'O'
	movwf TEMP_15
	goto wochentag_end

	movlw 0x05
	subwf TEMP_13,w
	btfss STATUS,Z
	goto $+6
	movlw A'F'
	movwf TEMP_14
	movlw A'R'
	movwf TEMP_15
	goto wochentag_end

	movlw 0x06
	subwf TEMP_13,w
	btfss STATUS,Z
	goto $+6
	movlw A'S'
	movwf TEMP_14
	movlw A'A'
	movwf TEMP_15
	goto wochentag_end

	movlw 0x07
	subwf TEMP_13,w
	btfss STATUS,Z
	goto $+6
	movlw A'S'
	movwf TEMP_14
	movlw A'O'
	movwf TEMP_15
	goto wochentag_end

	movlw A'F'				;bei Fehler
	movwf TEMP_14
	movlw A'F'
	movwf TEMP_15
wochentag_end
	return

disp_zahl
	movlw 0x8A					;Zeichen 11 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw DCF_BIT				;Daten fr Sekunden-/bitzhler
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr Sekunde 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr Sekunde 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	movfw DCF_COUNT				;Hole Zhler ins W-Register
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0xCC					;Zeichen 13 in der 2. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TBIT_H_TEMP			;Daten
	btfsc STATUS,Z				;prfen ob >0<
	goto $+3					;= 0, ersetze durch Blank
	addlw b'00110000'			;>0<, addiere obere 4 bit
	goto $+2					;
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	movfw TBIT_L_TEMP			;Daten fr Monat
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_13				;Daten 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return

disp_set_datum
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'D'					;ASCII:U
	call disp_daten				;bertrage das Zeichen
	movlw A'T'					;ASCII:h
	call disp_daten				;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movfw DCF_TAG				;Daten fr Tag
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	movwf TEMP_VAL_PREV			;speichern des Wertes im TEMP_VAL_PREV Register fr setzen des Cursors
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;Daten fr "."
	call disp_daten				;bertrage das Zeichen
	movfw DCF_MONAT				;Daten fr Monat
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;Daten fr "."
	call disp_daten				;bertrage das Zeichen
	movfw DCF_JAHR				;Daten fr Jahr
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movfw DCF_WTAG				;Daten fr Wochtagszahl
	movwf TEMP_13
	call wochentag				;Aufruf des Unterprogramms wochentag
	movfw TEMP_14				;Daten fr Wochentag, 1.Ziffer
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr Wochentag, 2.Ziffer
	call disp_daten				;bertrage das Zeichen	

	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'U'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'Z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw b'00111010'			;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movfw DCF_STD				;Daten fr Stunde
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movfw DCF_MIN				;Daten fr Minute
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;Daten fr ":"
	call disp_daten				;bertrage das Zeichen
	movfw SEK_TEMP				;Daten fr Sekunde
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'Z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'Z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw PM_TEM				;Vorzeichen +/-
	call disp_daten				;bertrage das Zeichen
	movfw DCF_MESZ				;Daten fr Zeitzone
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return

disp_set_standort
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'G'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'o'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'.'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0xC2					;Zeichen 3 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw GRD_TEM				;Daten fr Winkelgrad
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_13				;Daten fr 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0xDF					;Zeichen 
	call disp_daten				;bertrage das Zeichen
	movfw MIN_TEM				;Daten fr Winkelminuten
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw MIN_TEM_01			;Daten fr Winkelsekunden
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	btfss PM_TEM,0				;prfen ob PM_TEM = 1 (West)
	movlw A'E'					;PM_TEM = 0 (Ost)
	btfsc PM_TEM,0				;prfen ob PM_TEM = 0 (Ost)
	movlw A'W'					;PM_TEM = 1 (West)
	call disp_daten				;bertrage das Zeichen
	return

disp_set_dcfup_m
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'U'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'p'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'j'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'?'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'O'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'K'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'='					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'J'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'M'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'u'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'='					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'N'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_set_dcfup_a
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'U'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'p'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'd'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'z'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw PM_TEM				;Daten fr Updatezeit (Stunde)
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'h'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x8B					;Zeichen 12 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return	

disp_set_para1					;Anzeige Text im Menpunkt Parameter Auto
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'B'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A't'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'l'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'g'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'e'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A':'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'0'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

disp_set_para2					;Anzeige Daten im Menpunkt Parameter Auto
	movlw 0x8A					;Zeichen 11 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw TBIT_H_TEMP			;Daten fr Bitlnge, berlauf
	btfsc STATUS,Z				;prfen ob >0<
	goto $+3					;= 0, ersetze durch Blank
	addlw b'00110000'			;>0<, addiere obere 4 bit
	goto $+2					;
	movlw 0x20					;Blank
	call disp_daten				;bertrage das Zeichen
	movfw TBIT_L_TEMP			;Daten fr Bitlnge
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_13				;Daten 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw GRD_TEM				;Daten fr Zhler L-Bit
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw PM_TEM				;Daten fr Mittelwert L-Bit
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_13				;Daten 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movfw SEK_TEMP				;Daten fr Zhler H-Bit
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw REST_SEK_TEMP			;Daten fr Mittelwert H-Bit
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_13				;Daten 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	return

disp_set_para3
	movlw 0x80					;Zeichen 1 in der 1. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw MIN_TEM_01			;Daten fr Low Bit min
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_13				;Daten 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'L'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'x'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw PM_TEM				;Daten fr Low Bit max
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_13				;Daten 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw 0xC0					;Zeichen 1 in der 2. Zeile
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'i'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'n'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw STD_TEMP				;Daten fr High Bit min
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_13				;Daten 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movlw A'H'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'a'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'x'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movfw REST_SEK_TEMP			;Daten fr High Bit max
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 100er, 10er und 1er
	movfw TEMP_13				;Daten 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_14				;Daten 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten 1ser, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	return

disp_q1hz
	movlw 0x80					;Zeichen 1 in der 1. Zeile (=0x80 + 0x00)
	call disp_befehl			;bertrage das Kommando zum setzen der DDRAM-Adresse
	movfw JAHR_4				;Vorzeichen der Abweichung
	call disp_daten				;bertrage das Zeichen
	movfw STD_TEMP				;Abweichung 100/1000er
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr 1000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw MIN_TEMP				;Abweichung 1/10er
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A'p'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'p'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw A'm'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	movlw 0x20					;blank
	call disp_daten				;bertrage das Zeichen
	movfw SEK_TEMP				;Sekunden 100/1000er
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr 1000er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 100er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw REST_SEK_TEMP			;Sekunden 1/10er
	call convert_dig			;Aufruf des Unterprogramms zum Auftrennen in 10er und 1er
	movfw TEMP_14				;Daten fr 10er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movfw TEMP_15				;Daten fr 1er, untere 4 bit
	addlw b'00110000'			;addiere obere 4 bit
	call disp_daten				;bertrage das Zeichen
	movlw A's'					;ASCII:
	call disp_daten				;bertrage das Zeichen
	return

Init_display					;Unterprogramm zur Initialisierung des Displays
	movlw d'250'				;Wartezeit des Displays von 250ms
	movwf LOOP_1				;
	bcf PCLATH,3				;Select Page 0
	call wait 		
	bsf PCLATH,3				;Select Page 1
	movlw 0x38					;Befehl: Function Set (00111000)
	call disp_befehl	;
	movlw 0x38					;Befehl: Function Set (00111000), (muss zweimal bertragen werden)
	call disp_befehl	;
	movlw 0x14					;Befehl: Bias Set (00010100) (fr EA DOG-M162 Displays)
	call disp_befehl	;
	movlw 0x78					;Befehl: Contrast Set (01111000) (fr EA DOG-M162 Displays)
	call disp_befehl	;
	movlw 0x52					;Befehl: Power/Icon/Contrast Contr. (01010010) (fr EA DOG-M162 Displays)
	call disp_befehl	;
	movlw 0x69					;Befehl: Follower Control (01101001) (fr EA DOG-M162 Displays)
	call disp_befehl	;
	movlw d'250'				;Wartezeit des Displays von 250ms
	movwf LOOP_1				;
	bcf PCLATH,3				;Select Page 0
	call wait 		
	bsf PCLATH,3				;Select Page 1
	call cursor_off				;Aufruf des Unterprogramms zum Ausschalten des Cursors
	call clear_display			;Aufruf des Unterprogramms zum Lschen des Displays
	movlw 0x06					;Befehl: Entry Mode Set (00000110)
	call disp_befehl	;
	return

clear_display					;Unterprogramm zum lschen des Displays					
	movlw 0x01					;Befehl: Clear Display
	call disp_befehl	;
	movlw d'8'					;Wartezeit des Displays von 8ms
	movwf LOOP_1				;
	bcf PCLATH,3				;Select Page 0
	call wait 		
	bsf PCLATH,3				;Select Page 1
	return

cursor_off
	movlw 0x0C					;Befehl: Display On/Off Control, Cursor OFF (00001100)
	call disp_befehl	;
	movlw d'20'					;Wartezeit min 27us
	movwf LOOP_1				;
	decfsz LOOP_1				;
	goto $-1
	return

cursor_on
	movlw 0x0E					;Befehl: Display On/Off Control, Cursor ON (00001110)
	call disp_befehl	;
	movlw d'20'					;geforderte Wartezeit min 27us
	movwf LOOP_1				;
	decfsz LOOP_1				;
	goto $-1
	return


disp_befehl
	movwf DISP_SHIFT_REG		;Register mit den Daten, die ausgegeben werden sollen
	bcf DISP_RS					;RS auf Low setzen, da es sich um ein Kommando handelt
	call disp_routine			;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
	return

disp_daten
	movwf DISP_SHIFT_REG		;Das W-Register wird in das Reg. DISP_SHIFT_REG kopiert
	bsf DISP_RS					;RS auf High setzen, da es sich um ein Zeichen handelt
	call disp_routine			;Aufruf des Unterprogramms zum Schreiben der Display Adresse / Daten
	return

disp_routine
	bsf DISP_CLK				;=Enable
	nop
	nop
	movfw DISP_SHIFT_REG
	movwf PORTC
	nop
	nop
	bcf DISP_CLK				;=Enable, mit der negativen flanke werden daten bertragen
	movlw d'40'
	movwf LOOP_1
	decfsz LOOP_1
	goto $-1
	return

convert_dig					;Unterprogramm zur Umwandlung von zB Zahl 123 in 1,2,3
	movwf TEMP_17			;Temp Register fr Zwischenspeicherung des zu zerlegenden Wertes
	clrf TEMP_16			;lsche Temp Register
	clrf TEMP_13			;lsche Temp Register
loop_100er
	movfw TEMP_17
	movwf TEMP_16
	movlw d'100'			;schreibe Wert 100 ins W-Register
	subwf TEMP_17			;reduziere den zu zerlegenden Werte um 100
	btfss STATUS,C
	goto loop_100er_end
	incf TEMP_13			;Temp Register fr Zwischenspeicherung 100er
	goto loop_100er
loop_100er_end
	clrf TEMP_14			;lsche Temp Register fr Zwischenspeicherung 10er
loop_10er
	movfw TEMP_16
	movwf TEMP_15			;Temp Register fr Zwischenspeicherung 1er
	movlw d'10'
	subwf TEMP_16
	btfss STATUS,C
	goto loop_10er_end
	incf TEMP_14			;Temp Register fr Zwischenspeicherung 10er
	goto loop_10er
loop_10er_end
	return




;*******************************************************************************************
;	PAGE 2, 	bcf PCLATH,3				;Select Page 2
;				bsf PCLATH,4
;*******************************************************************************************
	ORG 0x1000					;PAGE 2	

;*******************************************************************************************
;	Unterprogramme zur Berechnung der lokalen Sternzeit aus Uhrzeit (UTC), Datum, und Position (Lnge O/W)
;*******************************************************************************************

sternzeit
;Testbereich Anfang
	;movlw d'12'			;laden der Werte in Register
	;movwf DCF_JAHR		;
	;movlw d'12'			;laden der Werte in Register
	;movwf DCF_MONAT		;
	;movlw d'31'			;laden der Werte in Register
	;movwf DCF_TAG		;
	;movlw d'22'			;laden der Werte in Register
	;movwf DCF_STD		;
	;movlw d'59'			;laden der Werte in Register
	;movwf DCF_MIN		;
	;movlw d'59'			;laden der Werte in Register
	;movwf SEK_UZ		;
	;movlw b'10000010'	;laden der Werte in Register
	;movwf DCF_MESZ		;
;Testbereich Ende
;Korrektur der lokalen Zeit auf UTC
	btfsc DCF_MESZ,7	;prfen ob Wert fr Zeitzone negativ ist
	goto korr_negativ	;
	movfw DCF_MESZ		;hole die aktuelle Stunde ins W-Register
	subwf DCF_STD,f		;subtrahiere die Stunden versus UTC (MEZ=1,MESZ=2)
	btfsc STATUS,C		;prfe ob Ergebniss negativ
	goto copy_base		;wenn nein, weiter bei copy_base, wenn ja, addiere 24 zu DCF_STD und reduziere die Tage um 1
	movlw d'24'
	addwf DCF_STD,f		;
	decfsz DCF_TAG		;reduziere Tag um 1 und prfe ob 0
	goto copy_base
	decfsz DCF_MONAT	;reduziere Monat um 1 und prfe ob 0
	goto $+4			;wenn nicht 0 weiter bei Rcksetzen des Tages
	movlw d'12'
	movwf DCF_MONAT
	decf DCF_JAHR		;wenn 0 reduziere Jahr um 1

;Ermittlung des Tages, der nach Reduzierung des Monats ins Tagesregister zurckgeschrieben wird
	call monats_tage	;
	movwf DCF_TAG
	goto copy_base

korr_negativ
	bcf DCF_MESZ,7		;lsche Vorzeichen Bit 7
	movfw DCF_MESZ		;hole die aktuelle Stunde ins W-Register
	addwf DCF_STD,f		;addiere die Stunden versus UTC
	movlw d'24'
	subwf DCF_STD,w		;
	btfss STATUS,C		;prfe ob Ergebniss >23
	goto copy_base		;wenn nein, weiter bei copy_base, wenn ja, subtrahiere 24 zu DCF_STD und erhhe die Tage um 1
	movwf DCF_STD
	call monats_tage	;hole die maximalen Tage des Monats ins W-Register
	subwf DCF_TAG,w		;subtrahiere den aktuellen Tag vom Maximalwert
	btfsc STATUS,Z		;prfe ob Maximalwert schon erreicht
	goto $+3			;Maximalwert ist erreicht => rcksetzen des Tags auf 1 und erhhen des Monats
	incf DCF_TAG		;Maximalwert ist noch nicht erreicht => erhhe Tag um 1
	goto copy_base
	movlw d'1'			;Maximalwert ist erreicht => rcksetzen des Tags auf 1
	movwf DCF_TAG
	incf DCF_MONAT		;erhhen des Monats um 1
	movlw d'13'			;prfen ob nach Erhhung der Monat = 13
	subwf DCF_MONAT,w
	btfss STATUS,Z		;
	goto copy_base		;Wert des Monats ist nicht 13 => weiter bei copy_base
	movlw d'1'			;Wert des Monats ist 13 => Rcksetzen des Monats auf 1 und Erhhung des Jahres
	movwf DCF_MONAT
	incf DCF_JAHR

copy_base
	movlw d'6'			;lade Sternzeit fr UTC 01.01.2000 00:00:00 = 06:39:52
	movwf STD_TEMP
	movlw d'39'
	movwf MIN_TEMP
	movlw d'52'
	movwf SEK_TEMP
	movlw d'27'
	movwf REST_SEK_TEMP
;ermittlung der Jahreszahl im Schaltjahrrythmus, Schaltjahr =0, SJ+1 =1, SJ+2 =2, SJ+3 =3
	clrf JAHR_4			;lsche Register
	btfsc DCF_JAHR,0	;prfen ob bit 0=1
	bsf JAHR_4,0		;setze bit0
	btfsc DCF_JAHR,1	;prfen ob bit 0=1
	bsf JAHR_4,1		;setze bit0
;Korrektur mit 1,85sek/Jahr seit Jahr 2000 = 185hundertstel sek = Jahre *185/100
	movfw DCF_JAHR
	movwf TEMP_15		;Low byte des 1.Multiplikanten
	clrf TEMP_16		;High byte des 1.Multiplikanten
	movlw d'185'
	movwf TEMP_17		;Low byte des 2.Multiplikanten
	clrf TEMP_18		;High byte des 2.Multiplikanten
	call Mul16			;das Ergebniss der Multiplikation steht in Registern TEMP_14/13/12/11

;Division durch 100, Die Werte fr Zhler H/M/L stehen noch in den Registern TEMP_13/12/11
	movlw d'100'		;lade NENNER_L mit Wert 100
	movwf TEMP_15		;
	clrf TEMP_16		;lsche NENNER_H (=0)
	call Div16			;das Ergebniss der Division steht in Registern TEMP_13/12/11, der Rest in TEMP_18/17

	movfw TEMP_17		;Rest_L
	addwf REST_SEK_TEMP,f
	call ueberlauf_rest_sek
	movfw TEMP_11		;ZAEHLER_L
	addwf SEK_TEMP,f
	call ueberlauf_sek

;Korrektur fr Schaltjahr (SJ) = 0, SJ+1 = 2min57,41sek, SJ+2 = 1min58,27sek, SJ+3 = 0min59,13sek
	movlw 0x01
	subwf JAHR_4,w
	btfss STATUS,Z
	goto year_02
	movlw d'41'
	addwf REST_SEK_TEMP,f
	call ueberlauf_rest_sek
	movlw d'57'
	addwf SEK_TEMP,f
	call ueberlauf_sek
	movlw d'2'
	addwf MIN_TEMP,f
	call ueberlauf_min	
	goto year_end
year_02
	movlw 0x02
	subwf JAHR_4,w
	btfss STATUS,Z
	goto year_03
	movlw d'27'
	addwf REST_SEK_TEMP,f
	call ueberlauf_rest_sek
	movlw d'58'
	addwf SEK_TEMP,f
	call ueberlauf_sek
	movlw d'1'
	addwf MIN_TEMP,f
	call ueberlauf_min	
	goto year_end
year_03
	movlw 0x03
	subwf JAHR_4,w
	btfss STATUS,Z
	goto year_end
	movlw d'13'
	addwf REST_SEK_TEMP,f
	call ueberlauf_rest_sek
	movlw d'59'
	addwf SEK_TEMP,f
	call ueberlauf_sek
year_end

;ermittlung der abgelaufenen Tage im laufenden Jahr
	clrf TAGE_H
month_01
	movlw d'1'
	subwf DCF_MONAT,w
	btfss STATUS,Z
	goto month_02
	clrf TAGE_L
	goto month_end
month_02
	movlw d'2'
	subwf DCF_MONAT,w
	btfss STATUS,Z
	goto month_03
	movlw d'31'
	movwf TAGE_L
	goto month_end
month_03
	movlw d'3'
	subwf DCF_MONAT,w
	btfss STATUS,Z
	goto month_04
	movlw d'59'			;59
	movwf TAGE_L
	goto month_end
month_04
	movlw d'4'
	subwf DCF_MONAT,w
	btfss STATUS,Z
	goto month_05
	movlw d'90'
	movwf TAGE_L
	goto month_end
month_05
	movlw d'5'
	subwf DCF_MONAT,w
	btfss STATUS,Z
	goto month_06
	movlw d'120'
	movwf TAGE_L
	goto month_end
month_06
	movlw d'6'
	subwf DCF_MONAT,w
	btfss STATUS,Z
	goto month_07
	movlw d'151'
	movwf TAGE_L
	goto month_end
month_07
	movlw d'7'
	subwf DCF_MONAT,w
	btfss STATUS,Z
	goto month_08
	movlw d'181'
	movwf TAGE_L
	goto month_end
month_08
	movlw d'8'
	subwf DCF_MONAT,w
	btfss STATUS,Z
	goto month_09
	movlw d'212'
	movwf TAGE_L
	goto month_end
month_09
	movlw d'9'
	subwf DCF_MONAT,w
	btfss STATUS,Z
	goto month_10
	movlw d'243'
	movwf TAGE_L
	goto month_end
month_10
	movlw d'10'
	subwf DCF_MONAT,w
	btfss STATUS,Z
	goto month_11
	movlw d'17'
	movwf TAGE_L
	bsf TAGE_H,0
	goto month_end
month_11
	movlw d'11'
	subwf DCF_MONAT,w
	btfss STATUS,Z
	goto month_12
	movlw d'48'
	movwf TAGE_L
	bsf TAGE_H,0
	goto month_end
month_12
	movlw d'12'
	subwf DCF_MONAT,w
	btfss STATUS,Z
	goto month_end
	movlw d'78'
	movwf TAGE_L
	bsf TAGE_H,0
month_end				;bei Schaltjahr und Monate Mrz bis Dezember mu 1 Tag addiert werden
	movfw JAHR_4
	btfss STATUS,Z
	goto add_tag
	movlw d'3'
	subwf DCF_MONAT,w
	btfsc STATUS,C
	incf TAGE_L,f
add_tag					;zu den Tagen der vollen Monate werden noch die Tage des aktuellen Monats addiert
	decf DCF_TAG,w
	addwf TAGE_L,f
	btfsc STATUS,C
	incf TAGE_H,f

;Ermittlung der Verschiebung aus den abgelaufenen vollen Tagen des aktuellen Jahres: =Tage *236,556sek
;dieses erfolgt in 2 Schritten: 1: Tage * 236sek, 2: Tage * 556msek
	movfw TAGE_L
	movwf TEMP_15		;Low byte des 1.Multiplikanten
	movfw TAGE_H
	movwf TEMP_16		;High byte des 1.Multiplikanten
	movlw d'236'		;1.Schritt, 256s
	movwf TEMP_17		;Low byte des 2.Multiplikanten
	clrf TEMP_18		;High byte des 2.Multiplikanten
	call Mul16			;das Ergebniss der Multiplikation steht in Registern TEMP_14/13/12/11

;Division durch 3600 fr volle Stunden. Die Werte fr Zhler H/M/L stehen noch in den Registern TEMP_13/12/11
	movlw d'16'			;lade NENNER_L mit Wert 16
	movwf TEMP_15		;
	movlw d'14'			;lade NENNER_H mit Wert 14
	movwf TEMP_16		;
	call Div16			;das Ergebniss der Division steht in Registern TEMP_13/12/11, der Rest in TEMP_18/17

	movfw TEMP_11		;ZAEHLER_L, die vollen Stunden werden addiert
	addwf STD_TEMP,f
	call ueberlauf_std
;Rest Division durch 60 fr volle Minuten.
	movfw TEMP_17		;Schiebe den Rest_L nach Zhler_L
	movwf TEMP_11
	movfw TEMP_18		;Schiebe den Rest_H nach Zhler_M
	movwf TEMP_12
	clrf TEMP_13		;lsche Zhler_H
	movlw d'60'			;lade NENNER_L mit Wert 60
	movwf TEMP_15		;
	clrf TEMP_16		;lsche Nenner_H
	call Div16			;das Ergebniss der Division steht in Registern TEMP_13/12/11, der Rest in TEMP_18/17

	movfw TEMP_11		;ZAEHLER_L, die vollen Minuten werden addiert
	addwf MIN_TEMP,f
	call ueberlauf_min

	movfw TEMP_17		;Rest_L, die Sekunden werden addiert
	addwf SEK_TEMP,f
	call ueberlauf_sek
;2.Schritt
	movfw TAGE_L		;
	movwf TEMP_15		;Low byte des 1.Multiplikanten
	movfw TAGE_H
	movwf TEMP_16		;High byte des 1.Multiplikanten
	movlw d'44'			;2.Schritt, 556ms
	movwf TEMP_17		;Low byte des 2.Multiplikanten
	movlw d'2'			;2.Schritt, 556ms
	movwf TEMP_18		;High byte des 2.Multiplikanten
	call Mul16			;das Ergebniss der Multiplikation steht in Registern TEMP_14/13/12/11

;Division durch 60.000ms fr volle Minuten. Die Werte fr Zhler H/M/L stehen noch in den Registern TEMP_13/12/11
	movlw d'96'			;lade NENNER_L mit Wert 96
	movwf TEMP_15		;
	movlw d'234'		;lade NENNER_H mit Wert 234
	movwf TEMP_16		;
	call Div16			;das Ergebniss der Division steht in Registern TEMP_13/12/11, der Rest in TEMP_18/17

	movfw TEMP_11		;ZAEHLER_L, die vollen Minuten werden addiert
	addwf MIN_TEMP,f
	call ueberlauf_min

;Rest Division durch 1000 fr volle Sekunden.
	movfw TEMP_17		;Schiebe den Rest_L nach Zhler_L
	movwf TEMP_11
	movfw TEMP_18		;Schiebe den Rest_H nach Zhler_M
	movwf TEMP_12
	clrf TEMP_13		;lsche Zhler_H
	movlw d'232'		;lade NENNER_L mit Wert 232
	movwf TEMP_15		;
	movlw d'3'			;lade NENNER_H mit Wert 3
	movwf TEMP_16		;
	call Div16			;das Ergebniss der Division steht in Registern TEMP_13/12/11, der Rest in TEMP_18/17

	movfw TEMP_11		;ZAEHLER_L, die vollen Sekunden werden addiert
	addwf SEK_TEMP,f
	call ueberlauf_sek

;Rest Division durch 10 zur Umwandlung von ms in hundertstel sek.
	movfw TEMP_17		;Schiebe den Rest_L nach Zhler_L
	movwf TEMP_11
	movfw TEMP_18		;Schiebe den Rest_H nach Zhler_M
	movwf TEMP_12
	clrf TEMP_13		;lsche Zhler_H
	movlw d'10'			;lade NENNER_L mit Wert 10
	movwf TEMP_15		;
	movlw d'0'			;lade NENNER_H mit Wert 0
	movwf TEMP_16		;
	call Div16			;das Ergebniss der Division steht in Registern TEMP_13/12/11, der Rest in TEMP_18/17

	movfw TEMP_11		;ZAEHLER_L, die hundertstel Sekunden werden addiert
	addwf REST_SEK_TEMP,f
	call ueberlauf_rest_sek

;Ermittlung der Verschiebung aus den abgelaufenen vollen Stunden des aktuellen Tages: =Stunden *9,85sek
	movfw DCF_STD		;hole die aktuelle Std ins w-Register und addiere sie	
	addwf STD_TEMP,f
	call ueberlauf_std
	movfw DCF_STD
	movwf TEMP_15		;Low byte des 1.Multiplikanten
	clrf TEMP_16		;lsche High byte des 1.Multiplikanten
	movlw d'217'		;985 hundertstel sek
	movwf TEMP_17		;Low byte des 2.Multiplikanten
	movlw d'3'			;985 hundertstel sek
	movwf TEMP_18		;High byte des 2.Multiplikanten
	call Mul16			;das Ergebniss der Multiplikation steht in Registern TEMP_14/13/12/11

;Division durch 6000 fr volle Minuten. Die Werte fr Zhler H/M/L stehen noch in den Registern TEMP_13/12/11
	movlw d'112'		;lade NENNER_L mit Wert 112
	movwf TEMP_15		;
	movlw d'23'			;lade NENNER_H mit Wert 23
	movwf TEMP_16		;
	call Div16			;das Ergebniss der Division steht in Registern TEMP_13/12/11, der Rest in TEMP_18/17

	movfw TEMP_11		;ZAEHLER_L, die vollen Minuten werden addiert
	addwf MIN_TEMP,f
	call ueberlauf_min

;Rest Division durch 100 zur Umwandlung von 100stel in volle sek.
	movfw TEMP_17		;Schiebe den Rest_L nach Zhler_L
	movwf TEMP_11
	movfw TEMP_18		;Schiebe den Rest_H nach Zhler_M
	movwf TEMP_12
	clrf TEMP_13		;lsche Zhler_H
	movlw d'100'		;lade NENNER_L mit Wert 100
	movwf TEMP_15		;
	movlw d'0'			;lade NENNER_H mit Wert 0
	movwf TEMP_16		;
	call Div16			;das Ergebniss der Division steht in Registern TEMP_13/12/11, der Rest in TEMP_18/17

	movfw TEMP_11		;ZAEHLER_L, die vollen Sekunden werden addiert
	addwf SEK_TEMP,f
	call ueberlauf_sek
	movfw TEMP_17		;Rest_L, die hundertstel Sekunden werden addiert
	addwf REST_SEK_TEMP,f
	call ueberlauf_rest_sek

;Ermittlung der Verschiebung aus den abgelaufenen vollen Minuten des aktuellen Tages: =Minuten *0,16sek
	movfw DCF_MIN		;hole die aktuelle Minute ins w-Register und addiere sie	
	addwf MIN_TEMP,f
	call ueberlauf_min
	movfw DCF_MIN
	movwf TEMP_15		;Low byte des 1.Multiplikanten
	clrf TEMP_16		;lsche High byte des 1.Multiplikanten
	movlw d'16'			;16 hundertstel sek
	movwf TEMP_17		;Low byte des 2.Multiplikanten
	movlw d'0'			;
	movwf TEMP_18		;High byte des 2.Multiplikanten
	call Mul16			;das Ergebniss der Multiplikation steht in Registern TEMP_14/13/12/11

;Division durch 100 fr volle Sekunden. Die Werte fr Zhler H/M/L stehen noch in den Registern TEMP_13/12/11
	movlw d'100'		;lade NENNER_L mit Wert 100
	movwf TEMP_15		;
	movlw d'0'			;lade NENNER_H mit Wert 0
	movwf TEMP_16		;
	call Div16			;das Ergebniss der Division steht in Registern TEMP_13/12/11, der Rest in TEMP_18/17

	movfw TEMP_11		;ZAEHLER_L, die vollen Minuten werden addiert
	addwf SEK_TEMP,f
	call ueberlauf_sek
	movfw TEMP_17		;Rest_L, die hundertstel Sekunden werden addiert
	addwf REST_SEK_TEMP,f
	call ueberlauf_rest_sek

	movfw SEK_UZ		;hole die aktuelle Sekunde ins w-Register und addiere sie	
	addwf SEK_TEMP,f
	call ueberlauf_sek


;Korrektur der lokalen Sternzeit mit der Position Lnge Ost/West
	bcf PCLATH,4		;select Page 0
	call read_position	;lade die Positionsdaten in die F-Register
	bsf PCLATH,4		;select Page 2
;Umrechnung von Winkelgrad in h und min: grad/15 = std, Rest x 4 = min
	movfw GRD_TEM		;Schiebe den Grad Wert nach Zhler_L
	movwf TEMP_11
	clrf TEMP_12		;lsche Zhler_M
	clrf TEMP_13		;lsche Zhler_H
	movlw d'15'			;lade NENNER_L mit Wert 15
	movwf TEMP_15		;
	movlw d'0'			;lade NENNER_H mit Wert 0
	movwf TEMP_16		;
	call Div16			;das Ergebniss der Division steht in Registern TEMP_13/12/11, der Rest in TEMP_18/17
	bcf STATUS,C		;lsche Carry bit, damit es beim rotieren nicht strt
	rlf TEMP_17,f		;Multipliziere den Rest L mit 4 (2xrlf =x2x2=x4)
	rlf TEMP_17,f		;

	btfsc PM_TEM,0		;prfen ob LSB = 1 (0=Ost, 1=West)
	goto pos_neg_grd
	movfw TEMP_11
	addwf STD_TEMP,f
	call ueberlauf_std

	movfw TEMP_17
	addwf MIN_TEMP,f
	call ueberlauf_min
	goto korr_pos_min

pos_neg_grd
	movfw TEMP_11
	subwf STD_TEMP,f
	call unterlauf_std

	movfw TEMP_17
	subwf MIN_TEMP,f
	call unterlauf_min

;Umrechnung von Winkelminuten in min und sek: w-min/15 = min, Rest x 4 = sek
korr_pos_min
	movfw MIN_TEM		;Schiebe den Winkelminuten Wert nach Zhler_L
	movwf TEMP_11
	clrf TEMP_12		;lsche Zhler_M
	clrf TEMP_13		;lsche Zhler_H
	movlw d'15'			;lade NENNER_L mit Wert 15
	movwf TEMP_15		;
	movlw d'0'			;lade NENNER_H mit Wert 0
	movwf TEMP_16		;
	call Div16			;das Ergebniss der Division steht in Registern TEMP_13/12/11, der Rest in TEMP_18/17
	bcf STATUS,C		;lsche Carry bit, damit es beim rotieren nicht strt
	rlf TEMP_17,f		;Multipliziere den Rest L mit 4 (2xrlf =x2x2=x4)
	rlf TEMP_17,f		;

	btfsc PM_TEM,0		;prfen ob LSB = 1 (0=+, 1=-)
	goto pos_neg_min
	movfw TEMP_11
	addwf MIN_TEMP,f
	call ueberlauf_min

	movfw TEMP_17
	addwf SEK_TEMP,f
	call ueberlauf_sek
	goto korr_pos_sek

pos_neg_min
	movfw TEMP_11
	subwf MIN_TEMP,f
	call unterlauf_min

	movfw TEMP_17
	subwf SEK_TEMP,f
	call unterlauf_sek

;Umrechnung von Winkelsekunden in sek und hundertstel sek: w-sek/15 = sek, Rest x 100/15 = hundertstel sek
korr_pos_sek
	movfw MIN_TEM_01	;Schiebe den Winkelsekunden Wert nach Zhler_L
	movwf TEMP_11
	clrf TEMP_12		;lsche Zhler_M
	clrf TEMP_13		;lsche Zhler_H
	movlw d'15'			;lade NENNER_L mit Wert 15
	movwf TEMP_15		;
	movlw d'0'			;lade NENNER_H mit Wert 0
	movwf TEMP_16		;
	call Div16			;das Ergebniss der Division steht in Registern TEMP_13/12/11, der Rest in TEMP_18/17

	btfsc PM_TEM,0		;prfen ob LSB = 1 (0=+, 1=-)
	goto pos_neg_sek
	movfw TEMP_11
	addwf SEK_TEMP,f
	call ueberlauf_sek
	goto korr_pos_hsek

pos_neg_sek
	movfw TEMP_11
	subwf SEK_TEMP,f
	call unterlauf_sek

;Rest x 100/15 = hundertstel sek
korr_pos_hsek
	movfw TEMP_17		;Multipliziere den Rest mit 100 und teile durch 15
	movwf TEMP_15		;Low byte des 1.Multiplikanten
	clrf TEMP_16		;lsche High byte des 1.Multiplikanten
	movlw d'100'		;100 hundertstel sek
	movwf TEMP_17		;Low byte des 2.Multiplikanten
	movlw d'0'			;
	movwf TEMP_18		;High byte des 2.Multiplikanten
	call Mul16			;das Ergebniss der Multiplikation steht in Registern TEMP_14/13/12/11

;Division durch 15 fr hundertstel sek. Die Werte fr Zhler H/M/L stehen noch in den Registern TEMP_13/12/11
	movlw d'15'			;lade NENNER_L mit Wert 15
	movwf TEMP_15		;
	movlw d'0'			;
	movwf TEMP_16		;
	call Div16			;das Ergebniss der Division steht in Registern TEMP_13/12/11, der Rest in TEMP_18/17

	btfsc PM_TEM,0		;prfen ob LSB = 1 (0=+, 1=-)
	goto pos_neg_hsek
	movfw TEMP_11		;
	addwf REST_SEK_TEMP,f
	call ueberlauf_rest_sek
	goto korr_pos_rest

pos_neg_hsek
	movfw TEMP_11		;
	subwf REST_SEK_TEMP,f
	call unterlauf_rest_sek

korr_pos_rest
	movlw d'50'			;prfen ob der Rest >50 ist
	subwf REST_SEK_TEMP,w
	btfss STATUS,C
	goto $+3
	incf SEK_TEMP,f
	call ueberlauf_sek


	movfw STD_TEMP		;kopieren der Temp Register in die Sternzeit Register
	movwf STD_SZ
	movfw MIN_TEMP		;kopieren der Temp Register in die Sternzeit Register
	movwf MIN_SZ
	movfw SEK_TEMP		;kopieren der Temp Register in die Sternzeit Register
	movwf SEK_SZ
		
	return				;Ende des "Sternzeit" Unterprogramms

;*******************************************************************************************
; Unterprogramme zum Unterprogramm "Sternzeit"
;*******************************************************************************************

monats_tage					;Unterprogramm zur Ermittlung der Tage eines Monats, Rcksprung mit Anzahl Tage in W
	movlw d'2'				;Ermittlung der Tage des Monats Februar
	subwf DCF_MONAT,w
	btfss STATUS,Z
	goto monats_tag_30		;Monat ist nicht Februar, weiter bei Abfrage der Monate 4,6,9,11
	btfsc DCF_JAHR,0		;Monat ist Februar, wenn Bits 0+1 des Jahres = 0 => Schaltjahr
	goto $+5				;Bit 0 = 1 => kein Schaltjahr
	btfsc DCF_JAHR,1	
	goto $+3				;Bit 1 = 1 => kein Schaltjahr
	movlw d'29'				;Bits 0+1 = 0 => Schaltjahr
	goto monats_tage_end
	movlw d'28'				;=> kein Schaltjahr
	goto monats_tage_end
monats_tag_30
	movlw d'4'				; Ermittlung der Tage des Monats, April
	subwf DCF_MONAT,w
	btfsc STATUS,Z
	goto mon_tag_30_end
	movlw d'6'				; Ermittlung der Tage des Monats, Juni
	subwf DCF_MONAT,w
	btfsc STATUS,Z
	goto mon_tag_30_end
	movlw d'9'				; Ermittlung der Tage des Monats, September
	subwf DCF_MONAT,w
	btfsc STATUS,Z
	goto mon_tag_30_end
	movlw d'11'				; Ermittlung der Tage des Monats, November
	subwf DCF_MONAT,w
	btfsc STATUS,Z
	goto mon_tag_30_end
	movlw d'31'				;Monat ist 1,3,5,7,8,10 oder 12 mit 31 Tagen
	goto monats_tage_end
mon_tag_30_end
	movlw d'30'
monats_tage_end
	return


ueberlauf_rest_sek			;Unterprogramm zur Prfung ob ein berlauf der Register vorhanden ist
	movlw d'100'
	subwf REST_SEK_TEMP,w
	btfss STATUS,C
	return
	movlw d'156'			; =-100
	addwf REST_SEK_TEMP,f
	incf SEK_TEMP
ueberlauf_sek
	movlw d'60'
	subwf SEK_TEMP,w
	btfss STATUS,C
	return
	movlw d'196'			; =-60
	addwf SEK_TEMP,f
	incf MIN_TEMP
ueberlauf_min
	movlw d'60'
	subwf MIN_TEMP,w
	btfss STATUS,C
	return
	movlw d'196'			; =-60
	addwf MIN_TEMP,f
	incf STD_TEMP
ueberlauf_std
	movlw d'24'
	subwf STD_TEMP,w
	btfss STATUS,C
	return
	movlw d'232'			; =-24
	addwf STD_TEMP,f
	return

unterlauf_rest_sek			;Unterprogramm zur Prfung ob ein negativer berlauf der Register vorhanden ist
	btfsc STATUS,C
	return
	movlw d'100'			;
	addwf REST_SEK_TEMP,f
	movlw 0xFF
	addwf SEK_TEMP,f
unterlauf_sek
	btfsc STATUS,C
	return
	movlw d'60'			;
	addwf SEK_TEMP,f
	movlw 0xFF
	addwf MIN_TEMP,f
unterlauf_min
	btfsc STATUS,C
	return
	movlw d'60'			;
	addwf MIN_TEMP,f
	movlw 0xFF
	addwf STD_TEMP,f
unterlauf_std
	btfsc STATUS,C
	return
	movlw d'24'			;
	addwf STD_TEMP,f
	return



;*******************************************************************************************
; Division eines 24 bit Wertes durch einen 16 bit Wert das Ergebnis wird in die
; Register des Zhlers gespeichert.
;
; Zhler / Nenner = Zhler + Rest 
;
; Verwendete Register:
; Zhler (24 bit) 	- ZAEHLER_H=TEMP_13, ZAEHLER_M=TEMP_12, ZAEHLER_L=TEMP_11
; Nenner (16 bit) 	- NENNER_H=TEMP_16, NENNER_L=TEMP_15
;
; Zhler (8 bit)	- LOOP_1
; Rest (16 bit)		- REST_H=TEMP_18, REST_L=TEMP_17
;
; Ergebnis (24 bit) - TEMP_13:TEMP_12:TEMP_11
;
;*******************************************************************************************
Div16
	clrf TEMP_18			;Rest von vorherigen Berechnungen lschen, TEMP_18/17=H/L
	clrf TEMP_17					
	movlw d'24'				;LOOP_1 mit 24 laden, da 24 bit im Zhler
	movwf LOOP_1
div16_loop
	rlf TEMP_11,F       	;den gesamtem Zhler um 1 nach links schieben, TEMP_13/12/11 = H/M/L
	rlf TEMP_12,F
	rlf TEMP_13,F

	rlf TEMP_17,F			;das Register fr den Rest um 1 nach links schieben
	rlf TEMP_18,F

	rlf LOOP_1,F			;das Carrybit wird vorbergehend im LOOP_1 gespeichert
									;dadurch kann man sich ein Register sparen	
	movf TEMP_15,W			;Nenner_L
	subwf TEMP_17,F			;Subtrahiere Nenner_L von Rest_L (REST_L-NENNER_L=REST_L)
	movf TEMP_16,W			;Nenner_H
	btfss STATUS, C
	incfsz TEMP_16,W		;erhhe Nenner_H
	subwf TEMP_18,W			;Subtrahiere Nenner_H von Rest_H (REST_H-NENNER_H=W-Reg)
	
	btfsc STATUS, C			;prfe, ob das Ergebnis positiv ist
	bsf LOOP_1,0			;wenn ja, setzte gespeichertes Carrybit in LOOP_1
	
	btfsc LOOP_1,0			;prfe das gespeicherte Carrybit in DIV_COUNTER
	goto div16_jump         ;wenn Carry=1 springe nach div16_jump
	
	movf TEMP_15,W			;Nenner_L,	Subtraktion wieder rckgngig machen, wenn das Ergebnis
	addwf TEMP_17,F			;REST_L,	negativ war und ein bertrag aufgetreten ist
	movf TEMP_18,W			;REST_H

div16_jump
	movwf TEMP_18			;Speichere den Rest vom W-Register in REST_H
	bcf STATUS,C			;Carrybit lschen und das gesicherte Carrybit in Register
	rrf LOOP_1,F			;LOOP_1 wieder zurcksichern.
	decfsz LOOP_1,F 		;LOOP_1 um 1 verringern
	goto div16_loop         ;wenn der Counter nicht null ist wird die nchste
	 
	rlf TEMP_11,F			;ZAEHLER_L		gesamter Zhler noch einmal nach links schieben, damit
	rlf TEMP_12,F			;ZAEHLER_M		das richtige Ergebnis im Zhler steht
	rlf TEMP_13,F			;ZAEHLER_H
	return
	
;*******************************************************************************************
; 16 bit Multiplikation
; Die Werte fr die Multiplikation mssen in den Registern TEMP_15 (LB,M1), TEMP_16 (HB,M1) 
; und TEMP_17 (LB,M2), TEMP_18 (HB,M2)
; Das Ergebnis steht dann in den Registern TEMP_14 bis TEMP_11
;*******************************************************************************************
Mul16
	clrf TEMP_14			;Lschen des vorherigen Ergebnisses TEMP_14 =bit24-31
	clrf TEMP_13			;TEMP_13 =bit16-23
	clrf TEMP_12			;TEMP_12 =bit8-15
	clrf TEMP_11			;TEMP_11 =bit0-7
	bsf TEMP_12,7			;um zu erkennen, wann 16 mal geschoben wurde
mul16_loop
	rrf TEMP_16,F			;verschiebe High Byte Multiplikant1 um 1 nach rechts
	rrf TEMP_15,F			;verschiebe Low Byte Multiplikant1 um 1 nach rechts, wenn das LSB eine 1 ist wird das Carry gesetzt und es muss
	btfss STATUS,C			;eine Addition durchgefhrt werden ist das Carry = 0 entspricht
	goto mul16_rotate		;dies einer Multiplikation mit 0 und es wird nichts addiert
	movf TEMP_17,W			;Beginn der 16 bit Addition: Hole Low Byte Multiplikant2 ins W-Register
	addwf TEMP_13,F			;addiere bits 16-23 des Ergebnis Registers
	movf TEMP_18,W			;Hole High Byte Multiplikant2 ins W-Register
	btfsc STATUS,C			;prft, ob ein bertrag vorhanden ist
	incfsz TEMP_18,W		;es ist ein bertrag vorhanden und das Highbyte von Multiplikant2 wird um 1 erhht
	addwf TEMP_14,F			;kein bertrag, das Highbyte wird addiert
mul16_rotate
	rrf TEMP_14,F			;Das 32 Bit Ergebnisregister wird um 1 nach rechts geschoben
	rrf TEMP_13,F
	rrf TEMP_12,F
	rrf TEMP_11,F
	btfss STATUS,C			;ist das Bit 7 von MUL_E_2 im Carry angekommen wurde das Ergebnis
	goto mul16_loop			;insgesamt 16 mal rotiert und die Berechnung ist beendet

	return                  ; fertig



	END                	; Ende des Programms